"use strict";Object.defineProperty(exports, "__esModule", {value: true}); function _nullishCoalesce(lhs, rhsFn) { if (lhs != null) { return lhs; } else { return rhsFn(); } }var _cleaners = require('cleaners');


const asCoincapResponse = _cleaners.asObject.call(void 0, {
  data: _cleaners.asArray.call(void 0, 
    _cleaners.asObject.call(void 0, {
      symbol: _cleaners.asString,
      priceUsd: _cleaners.asString
    })
  )
})

const asCoincapError = _cleaners.asObject.call(void 0, {
  error: _cleaners.asOptional.call(void 0, _cleaners.asString),
  timestamp: _cleaners.asNumber
})

const asCoincapAssets = _cleaners.asArray.call(void 0, 
  _cleaners.asObject.call(void 0, {
    id: _cleaners.asString,
    symbol: _cleaners.asString
  })
)







const currencyMap = {}

 function makeCoincapPlugin(opts) {
  const { io, log } = opts
  const { fetchCors = io.fetch } = io

  return {
    rateInfo: {
      pluginId: 'coincap',
      displayName: 'Coincap'
    },

    async fetchRates(pairsHint) {
      const pairs = []
      // Create unique ID map
      if (Object.keys(currencyMap).length === 0) {
        const assets = await fetchCors(`https://api.coincap.io/v2/assets/`)
        const assetsJson = await assets.json()
        const assetIds = asCoincapAssets(assetsJson.data)
        assetIds.forEach(code => (currencyMap[code.symbol] = code.id))
      }

      // Create query strings
      const queryStrings = []
      let filteredPairs = []
      for (let i = 0; i < pairsHint.length; i++) {
        if (
          currencyMap[pairsHint[i].fromCurrency] === '' ||
          currencyMap[pairsHint[i].fromCurrency] == null
        )
          continue
        if (pairsHint[i].fromCurrency.includes('iso:')) continue
        if (
          filteredPairs.some(
            cc => cc === currencyMap[pairsHint[i].fromCurrency]
          )
        )
          continue
        filteredPairs.push(currencyMap[pairsHint[i].fromCurrency])
        if (filteredPairs.length === 100 || i === pairsHint.length - 1) {
          queryStrings.push(filteredPairs.join(','))
          filteredPairs = []
        }
      }

      for (const query of queryStrings) {
        // Coincap only provides prices in USD
        try {
          const reply = await fetch(
            `https://api.coincap.io/v2/assets?ids=${query}`
          )
          const json = await reply.json()
          const { error } = asCoincapError(json)
          if ((error != null && error !== '') || !reply.ok) {
            throw new Error(
              `CoincapHistorical returned code ${JSON.stringify(
                _nullishCoalesce(error, () => ( reply.status))
              )}`
            )
          }
          asCoincapResponse(json).data.forEach(rate =>
            pairs.push({
              fromCurrency: rate.symbol,
              toCurrency: 'iso:USD',
              rate: Number(rate.priceUsd)
            })
          )
        } catch (e) {
          log.warn(`Issue with Coincap rate data structure ${String(e)}`)
        }
      }
      return pairs
    }
  }
} exports.makeCoincapPlugin = makeCoincapPlugin;
