"use strict";Object.defineProperty(exports, "__esModule", {value: true}); function _nullishCoalesce(lhs, rhsFn) { if (lhs != null) { return lhs; } else { return rhsFn(); } }var _cleaners = require('cleaners');






const asInitOptions = _cleaners.asObject.call(void 0, {
  apiKey: _cleaners.asString
})

const asRates = _cleaners.asObject.call(void 0, _cleaners.asNumber)

const asCurrencyConverterResponse = _cleaners.asObject.call(void 0, {
  status: _cleaners.asOptional.call(void 0, _cleaners.asNumber),
  error: _cleaners.asOptional.call(void 0, _cleaners.asString)
}).withRest

const checkAndPush = (isoCc, ccArray) => {
  if (isoCc !== 'iso:USD' && isoCc.slice(0, 4) === 'iso:') {
    const cc = isoCc.slice(4).toUpperCase()
    if (!ccArray.includes(`USD_${cc}`)) {
      ccArray.push(`USD_${cc}`)
    }
  }
}

 function makeCurrencyconverterapiPlugin(
  opts
) {
  const { io, log } = opts
  const { fetchCors = io.fetch } = io
  const { apiKey } = asInitOptions(opts.initOptions)

  return {
    rateInfo: {
      pluginId: 'currencyconverterapi',
      displayName: 'CurrencyConverterAPI'
    },

    async fetchRates(pairsHint) {
      pairsHint = pairsHint.concat([
        { fromCurrency: 'iso:USD', toCurrency: 'iso:IMP' },
        { fromCurrency: 'iso:USD', toCurrency: 'iso:IRR' }
      ])
      const isoCodesWanted = []
      for (const pair of pairsHint) {
        checkAndPush(pair.fromCurrency, isoCodesWanted)
        checkAndPush(pair.toCurrency, isoCodesWanted)
      }

      const pairs = []
      const query = isoCodesWanted.join(',')
      try {
        const response = await fetchCors(
          `https://api.currconv.com/api/v7/convert?q=${query}&compact=ultra&apiKey=${apiKey}`
        )
        const { status, error, ...rest } = asCurrencyConverterResponse(
          await response.json()
        )
        const rates = rest
        if (
          (status != null && status !== 200) ||
          (error != null && error !== '') ||
          !response.ok
        ) {
          throw new Error(
            `CurrencyConvertor returned with status: ${JSON.stringify(
              _nullishCoalesce(status, () => ( response.status))
            )} and error: ${JSON.stringify(error)}`
          )
        }
        for (const rate of Object.keys(asRates(rates))) {
          pairs.push({
            fromCurrency: 'iso:USD',
            toCurrency: `iso:${rate.split('_')[1]}`,
            rate: rates[rate]
          })
        }
      } catch (e) {
        log.warn(
          `Failed to get ${query} from currencyconverterapi.com`,
          e.message
        )
      }
      return pairs
    }
  }
} exports.makeCurrencyconverterapiPlugin = makeCurrencyconverterapiPlugin;
