"use strict";Object.defineProperty(exports, "__esModule", {value: true}); function _nullishCoalesce(lhs, rhsFn) { if (lhs != null) { return lhs; } else { return rhsFn(); } }var _cleaners = require('cleaners');






const asInitOptions = _cleaners.asObject.call(void 0, {
  apiKey: _cleaners.asString
})

const asNomicsResponse = _cleaners.asArray.call(void 0, 
  _cleaners.asObject.call(void 0, {
    price: _cleaners.asOptional.call(void 0, _cleaners.asString),
    symbol: _cleaners.asString
  })
)

const UNIQUE_ID_MAP = {
  BOO: 'BOO4'
}

 function makeNomicsPlugin(opts) {
  const { io, log } = opts
  const { fetchCors = io.fetch } = io
  const { apiKey } = asInitOptions(opts.initOptions)

  if (apiKey == null) {
    throw new Error('No Nomics exchange rates API key provided')
  }
  return {
    rateInfo: {
      pluginId: 'nomics',
      displayName: 'Nomics'
    },

    async fetchRates(pairsHint) {
      const pairs = []

      // Create query strings
      const queryStrings = []
      let filteredPairs = []
      for (let i = 0; i < pairsHint.length; i++) {
        if (pairsHint[i].fromCurrency.includes('iso:')) continue
        if (filteredPairs.some(cc => cc === pairsHint[i].fromCurrency)) continue
        filteredPairs.push(
          _nullishCoalesce(UNIQUE_ID_MAP[pairsHint[i].fromCurrency], () => ( pairsHint[i].fromCurrency))
        )
        if (filteredPairs.length === 100 || i === pairsHint.length - 1) {
          queryStrings.push(filteredPairs.join(','))
          filteredPairs = []
        }
      }

      for (const query of queryStrings) {
        try {
          const reply = await fetchCors(
            `https://api.nomics.com/v1/currencies/ticker?key=${apiKey}&ids=${query}&convert=USD`
          )
          if (reply.status === 429 || reply.status === 401 || !reply.ok)
            throw new Error(`Nomics returned with status: ${reply.status}`)
          asNomicsResponse(await reply.json()).forEach(rate => {
            // When Nomics considers a coin "dead" they don't return a price
            if (rate.price != null)
              pairs.push({
                fromCurrency: rate.symbol,
                toCurrency: 'iso:USD',
                rate: Number(rate.price)
              })
          })
        } catch (e) {
          log.warn(
            `Issue with Nomics rate data structure. ${String(e.message)}`
          )
        }
      }
      return pairs
    }
  }
} exports.makeNomicsPlugin = makeNomicsPlugin;
