import { EdgeCurrencyWallet, EdgeSpendInfo, EdgeSwapInfo, EdgeSwapQuote, EdgeSwapRequest, EdgeTransaction, JsonObject } from 'edge-core-js/types';
import { MakeTxParams } from './swap/defi/xrp/xrpDexTypes';
import { EdgeSwapRequestPlugin } from './swap/types';
/**
 * Ensures that a date is in the future by at least the given amount.
 */
export declare function ensureInFuture(date?: Date, marginSeconds?: number): Date | undefined;
interface SwapOrderSpendInfo {
    spendInfo: EdgeSpendInfo;
}
interface SwapOrderMakeTx {
    makeTxParams: MakeTxParams;
}
declare type SwapOrderInner = SwapOrderMakeTx | SwapOrderSpendInfo;
export declare type SwapOrder = SwapOrderInner & {
    canBePartial?: boolean;
    maxFulfillmentSeconds?: number;
    request: EdgeSwapRequest;
    swapInfo: EdgeSwapInfo;
    fromNativeAmount: string;
    expirationDate?: Date;
    preTx?: EdgeTransaction;
    metadataNotes?: string;
};
export declare function makeSwapPluginQuote(order: SwapOrder): Promise<EdgeSwapQuote>;
export declare const getMaxSwappable: <T extends any[]>(fetchSwap: (request: EdgeSwapRequestPlugin, ...args: T) => Promise<SwapOrder>, request: EdgeSwapRequestPlugin, ...args: T) => Promise<EdgeSwapRequestPlugin>;
export declare const customFeeCacheMap: {
    [uid: string]: {
        customNetworkFee?: JsonObject;
        timestamp: number;
    };
};
export declare const customFeeCache: {
    createUid: () => string;
    getFees: (uid: string) => JsonObject | undefined;
    setFees: (uid: string, customNetworkFee?: JsonObject) => void;
};
interface AllCodes {
    fromMainnetCode: string;
    toMainnetCode: string;
    fromCurrencyCode: string;
    toCurrencyCode: string;
}
export declare const getCodes: (request: EdgeSwapRequestPlugin) => AllCodes;
export interface InvalidCurrencyCodes {
    from: {
        [pluginId: string]: 'allCodes' | 'allTokens' | string[];
    };
    to: {
        [pluginId: string]: 'allCodes' | 'allTokens' | string[];
    };
}
/**
 * Throws if either currency code has been disabled by the plugin
 */
export declare function checkInvalidCodes(invalidCodes: InvalidCurrencyCodes, request: EdgeSwapRequestPlugin, swapInfo: EdgeSwapInfo): void;
export interface CurrencyCodeTranscriptions {
    [code: string]: {
        [code: string]: string;
    };
}
export interface MainnetPluginIdTranscriptionMap {
    [pluginId: string]: string;
}
export interface CurrencyCodeTranscriptionMap {
    [pluginId: string]: {
        [currencyCode: string]: string;
    };
}
/**
 * Returns all four codes with transcription
 */
export declare const getCodesWithTranscription: (request: EdgeSwapRequestPlugin, mainnetTranscriptionMap: MainnetPluginIdTranscriptionMap, currencyCodeTranscriptionMap?: CurrencyCodeTranscriptionMap) => AllCodes;
export declare const isLikeKind: (fromCurrencyCode: string, toCurrencyCode: string) => boolean;
export declare const getTokenId: (coreWallet: EdgeCurrencyWallet, currencyCode: string) => string | undefined;
export declare const consify: (val: any) => void;
export {};
