/**
 * Routines copied from https://github.com/florent-uzio/xrpl.js-demo.git
 */
import { BookOfferCurrency, BookOffersRequest } from 'xrpl';
import { BookOffersResponse } from 'xrpl/dist/npm/models/methods/bookOffers';
import { MethodOptions } from './xrpDexTypes';
/**
 * Convert an hexadecimal value to readable string.
 *
 * @param hex The hexadecimal to convert.
 * @returns A human readable string.
 */
export declare const hexToString: (hex: string) => string;
/**
 * Helper to correctly display the currency code if its length is more than 3.
 * Example: 5553444D00000000000000000000000000000000 will become USDM
 *
 * @param currencyCode The currency code to potentially format correctly.
 * @returns A {@link String} representing the currency code readable by a human.
 */
export declare const convertHexCurrencyCodeToString: (currencyCode: string) => string;
export declare const getBookOffers: ({ taker_gets, taker_pays, ...rest }: BookOffersRequest, { client, showLogs }: MethodOptions) => Promise<BookOffersResponse>;
export declare type GetBuyQuoteProps = Omit<BookOffersRequest, 'taker_gets' | 'taker_pays' | 'command'> & {
    /**
     * The currency we want to buy.
     * If the currency is an IOU, the issuer needs to be mentioned.
     */
    weWant: BookOfferCurrency;
    /**
     * The amount of token we want to buy.
     */
    weWantAmountOfToken: number;
    /**
     * The counter currency.
     */
    counterCurrency: BookOfferCurrency;
};
/**
 * Function to get a quote of a token to buy.
 * This is an experimental function and must not be used in production without checking it does what you need.
 *
 * @param {Object} props The props to pass to the function.
 * @param {TakerAmount} props.weWant The token we want to acquire. Specify the currency and optionaly the issuer (if the currency is not XRP).
 * @param {number} props.weWantAmountOfToken The amount of token we want to acquire.
 * @param {string} props.taker (Optional) The Address of an account to use as a perspective. The response includes this account's Offers even if they are unfunded.
 * @param {TakerAmount} props.counterCurrency The counter currency.
 * @returns void, display a message regarding the result of the quote.
 */
export declare const getBuyQuote: ({ weWant, weWantAmountOfToken, counterCurrency, ...rest }: GetBuyQuoteProps, { client, showLogs }: MethodOptions) => Promise<number>;
declare type GetSellQuoteProps = Omit<BookOffersRequest, 'taker_gets' | 'taker_pays' | 'command'> & {
    /**
     * The currency we want to sell.
     * If the currency is an IOU, the issuer needs to be mentioned.
     */
    weSell: BookOfferCurrency;
    /**
     * The amount of currency we want to sell.
     */
    weSellAmountOfTokens: number;
    /**
     * The counter currency.
     */
    counterCurrency: BookOfferCurrency;
};
/**
 * Function to get a quote of a token to sell.
 * The quote will give you the amount of the counter token that you can expect to get from that sell.
 * This is an experimental function and must not be used in production without checking it does what you need.
 *
 * @param {Object} props The props to pass to the function.
 * @param {TakerAmount} props.weSell The token we want to sell. Specify the currency and optionaly the issuer (if the currency is not XRP).
 * @param {number} props.weSellAmountOfTokens The amount of token we want to sell.
 * @param {string} props.taker (Optional) The Address of an account to use as a perspective. The response includes this account's Offers even if they are unfunded.
 * @param {TakerAmount} props.counterCurrency The counter currency.
 * @returns void, display a message regarding the result of the quote.
 */
export declare const getSellQuote: ({ weSell, weSellAmountOfTokens, counterCurrency, ...rest }: GetSellQuoteProps, { client, showLogs }: MethodOptions) => Promise<number>;
export {};
