"use strict";Object.defineProperty(exports, "__esModule", {value: true}); function _nullishCoalesce(lhs, rhsFn) { if (lhs != null) { return lhs; } else { return rhsFn(); } } function _optionalChain(ops) { let lastAccessLHS = undefined; let value = ops[0]; let i = 1; while (i < ops.length) { const op = ops[i]; const fn = ops[i + 1]; i += 2; if ((op === 'optionalAccess' || op === 'optionalCall') && value == null) { return undefined; } if (op === 'access' || op === 'optionalAccess') { lastAccessLHS = value; value = fn(value); } else if (op === 'call' || op === 'optionalCall') { value = fn((...args) => value.call(lastAccessLHS, ...args)); lastAccessLHS = undefined; } } return value; }var _biggystring = require('biggystring');












var _types = require('edge-core-js/types');




const likeKindAssets = [
  ['BTC', 'WBTC', 'SBTC', 'RBTC'],
  ['ETH', 'WETH'],
  ['USDC', 'USDT', 'DAI']
]

/**
 * Ensures that a date is in the future by at least the given amount.
 */
 function ensureInFuture(
  date,
  marginSeconds = 30
) {
  if (date == null) return
  const target = Date.now() + marginSeconds * 1000
  return target < date.valueOf() ? date : new Date(target)
} exports.ensureInFuture = ensureInFuture;






















 async function makeSwapPluginQuote(
  order
) {
  const {
    canBePartial,
    maxFulfillmentSeconds,
    fromNativeAmount,
    request,
    swapInfo,
    expirationDate,
    preTx,
    metadataNotes
  } = order
  const { fromWallet } = request

  let tx
  let swapData
  if ('spendInfo' in order) {
    const { spendInfo } = order
    swapData = spendInfo.swapData
    tx = await fromWallet.makeSpend(spendInfo)
  } else {
    const { makeTxParams } = order
    swapData = makeTxParams.swapData
    tx = await fromWallet.otherMethods.makeTx(makeTxParams)
  }
  const toNativeAmount = _optionalChain([swapData, 'optionalAccess', _ => _.payoutNativeAmount])
  const destinationAddress = _optionalChain([swapData, 'optionalAccess', _2 => _2.payoutAddress])
  const isEstimate = _nullishCoalesce(_optionalChain([swapData, 'optionalAccess', _3 => _3.isEstimate]), () => ( false))
  const quoteId = _optionalChain([swapData, 'optionalAccess', _4 => _4.orderId])
  if (
    fromNativeAmount == null ||
    toNativeAmount == null ||
    destinationAddress == null
  ) {
    throw new Error(
      `Invalid makeSwapPluginQuote args from ${swapInfo.pluginId}`
    )
  }

  let nativeAmount =
    tx.parentNetworkFee != null ? tx.parentNetworkFee : tx.networkFee

  if (preTx != null)
    nativeAmount = _biggystring.add.call(void 0, 
      nativeAmount,
      preTx.parentNetworkFee != null ? preTx.parentNetworkFee : preTx.networkFee
    )

  const out = {
    canBePartial,
    maxFulfillmentSeconds,
    request,
    swapInfo,
    fromNativeAmount,
    toNativeAmount,
    networkFee: {
      currencyCode: fromWallet.currencyInfo.currencyCode,
      nativeAmount
    },
    pluginId: swapInfo.pluginId,
    expirationDate,
    isEstimate,
    async approve(opts) {
      if (preTx != null) {
        const signedTransaction = await fromWallet.signTx(preTx)
        const broadcastedTransaction = await fromWallet.broadcastTx(
          signedTransaction
        )
        await fromWallet.saveTx(broadcastedTransaction)
      }
      tx.metadata = { ...(_nullishCoalesce(_optionalChain([opts, 'optionalAccess', _5 => _5.metadata]), () => ( {}))), ...tx.metadata }
      if (metadataNotes != null) {
        tx.metadata.notes = `${metadataNotes}\n\n` + (_nullishCoalesce(tx.metadata.notes, () => ( '')))
      }

      const signedTransaction = await fromWallet.signTx(tx)
      const broadcastedTransaction = await fromWallet.broadcastTx(
        signedTransaction
      )
      await fromWallet.saveTx(signedTransaction)

      return {
        transaction: broadcastedTransaction,
        orderId: quoteId,
        destinationAddress
      }
    },

    async close() {}
  }
  return out
} exports.makeSwapPluginQuote = makeSwapPluginQuote;

 const getMaxSwappable = async ( 
  fetchSwap,
  request,
  ...args
) => {
  if (request.quoteFor !== 'max') return request

  const requestCopy = { ...request }
  const { fromWallet, fromCurrencyCode } = requestCopy

  // First attempt a swap that uses the entire balance
  const balance = _nullishCoalesce(fromWallet.balances[fromCurrencyCode], () => ( '0'))
  requestCopy.nativeAmount = balance
  requestCopy.quoteFor = 'from'
  const swapOrder = await fetchSwap(requestCopy, ...args)
  if (!('spendInfo' in swapOrder)) {
    return request
  }

  // Then use getMaxSpendable with the partner's address
  delete swapOrder.spendInfo.spendTargets[0].nativeAmount
  let maxAmount = await fromWallet.getMaxSpendable(swapOrder.spendInfo)

  // Subtract fee from pretx
  if (
    swapOrder.preTx != null &&
    fromCurrencyCode === fromWallet.currencyInfo.currencyCode
  ) {
    maxAmount = _biggystring.sub.call(void 0, maxAmount, swapOrder.preTx.networkFee)
  }

  // Update and return the request object
  requestCopy.nativeAmount = maxAmount
  return requestCopy
}; exports.getMaxSwappable = getMaxSwappable

// Store custom fees so a request can use consistent fees when calling makeSpend multiple times
 const customFeeCacheMap

 = {}; exports.customFeeCacheMap = customFeeCacheMap

let swapId = '0'
 const customFeeCache = {
  createUid: () => {
    swapId = _biggystring.add.call(void 0, swapId, '1')
    exports.customFeeCacheMap[swapId] = { timestamp: Date.now() }
    return swapId
  },
  getFees: (uid) => {
    return _optionalChain([exports.customFeeCacheMap, 'optionalAccess', _6 => _6[uid], 'optionalAccess', _7 => _7.customNetworkFee])
  },
  setFees: (uid, customNetworkFee) => {
    for (const id of Object.keys(exports.customFeeCacheMap)) {
      if (Date.now() > exports.customFeeCacheMap[id].timestamp + 30000) {
        delete exports.customFeeCacheMap[id] // eslint-disable-line
      }
    }
    exports.customFeeCacheMap[uid] = { customNetworkFee, timestamp: Date.now() }
  }
}; exports.customFeeCache = customFeeCache








 const getCodes = (request) => ({
  fromMainnetCode: request.fromWallet.currencyInfo.currencyCode,
  toMainnetCode: request.toWallet.currencyInfo.currencyCode,
  fromCurrencyCode: request.fromCurrencyCode,
  toCurrencyCode: request.toCurrencyCode
}); exports.getCodes = getCodes

const getPluginIds = (
  request
) => ({
  fromPluginId: request.fromWallet.currencyInfo.pluginId,
  toPluginId: request.toWallet.currencyInfo.pluginId
})






const defaultInvalidCodes = {
  from: { ethereum: ['REP'] },
  to: { ethereum: ['REP'] }
}

/**
 * Throws if either currency code has been disabled by the plugin
 */
 function checkInvalidCodes(
  invalidCodes,
  request,
  swapInfo
) {
  const { fromPluginId, toPluginId } = getPluginIds(request)
  const {
    fromMainnetCode,
    toMainnetCode,
    fromCurrencyCode,
    toCurrencyCode
  } = exports.getCodes.call(void 0, request)

  const isSameAsset = (request) =>
    request.fromWallet.currencyInfo.pluginId ===
      request.toWallet.currencyInfo.pluginId &&
    request.fromCurrencyCode === request.toCurrencyCode

  function check(
    codeMap,
    direction,
    pluginId,
    main,
    token
  ) {
    const codes = codeMap[direction][pluginId]
    if (codes == null) return false
    if (codes === 'allCodes') return true
    if (codes === 'allTokens') return main !== token
    return codes.some(code => code === token)
  }

  if (
    check(
      invalidCodes,
      'from',
      fromPluginId,
      fromMainnetCode,
      fromCurrencyCode
    ) ||
    check(
      defaultInvalidCodes,
      'from',
      fromPluginId,
      fromMainnetCode,
      fromCurrencyCode
    ) ||
    check(invalidCodes, 'to', toPluginId, toMainnetCode, toCurrencyCode) ||
    check(
      defaultInvalidCodes,
      'to',
      toPluginId,
      toMainnetCode,
      toCurrencyCode
    ) ||
    isSameAsset(request)
  )
    throw new (0, _types.SwapCurrencyError)(
      swapInfo,
      request.fromCurrencyCode,
      request.toCurrencyCode
    )
} exports.checkInvalidCodes = checkInvalidCodes;

















const defaultCurrencyCodeTranscriptionMap = {
  ethereum: {
    REPV2: 'REP'
  }
}

const defaultMainnetTranscriptionMap = {
  optimism: 'OP', // mainnet code is ETH
  zksync: 'ZKSYNC' // mainnet code is also ETH
}

/**
 * Returns all four codes with transcription
 */
 const getCodesWithTranscription = (
  request,
  mainnetTranscriptionMap,
  currencyCodeTranscriptionMap = {}
) => {
  const {
    fromCurrencyCode,
    toCurrencyCode,
    fromMainnetCode,
    toMainnetCode
  } = exports.getCodes.call(void 0, request)

  for (const pluginId of Object.keys(defaultCurrencyCodeTranscriptionMap)) {
    if (currencyCodeTranscriptionMap[pluginId] == null)
      currencyCodeTranscriptionMap[pluginId] =
        defaultCurrencyCodeTranscriptionMap[pluginId]
    else
      currencyCodeTranscriptionMap[pluginId] = {
        ...defaultCurrencyCodeTranscriptionMap[pluginId],
        ...currencyCodeTranscriptionMap[pluginId]
      }
  }

  for (const pluginId of Object.keys(defaultMainnetTranscriptionMap)) {
    if (mainnetTranscriptionMap[pluginId] == null)
      mainnetTranscriptionMap[pluginId] =
        defaultMainnetTranscriptionMap[pluginId]
  }

  return {
    fromMainnetCode:
      _nullishCoalesce(mainnetTranscriptionMap[request.fromWallet.currencyInfo.pluginId], () => (
      fromMainnetCode)),
    toMainnetCode:
      _nullishCoalesce(mainnetTranscriptionMap[request.toWallet.currencyInfo.pluginId], () => (
      toMainnetCode)),
    fromCurrencyCode:
      _nullishCoalesce(_optionalChain([currencyCodeTranscriptionMap, 'access', _8 => _8[request.fromWallet.currencyInfo.pluginId], 'optionalAccess', _9 => _9[
        fromCurrencyCode
      ]]), () => ( fromCurrencyCode)),
    toCurrencyCode:
      _nullishCoalesce(_optionalChain([currencyCodeTranscriptionMap, 'access', _10 => _10[request.toWallet.currencyInfo.pluginId], 'optionalAccess', _11 => _11[
        toCurrencyCode
      ]]), () => ( toCurrencyCode))
  }
}; exports.getCodesWithTranscription = getCodesWithTranscription

 const isLikeKind = (
  fromCurrencyCode,
  toCurrencyCode
) => {
  // Check if the swap is Like Kind
  for (const assetList of likeKindAssets) {
    const matchFrom = assetList.find(cc => cc === fromCurrencyCode)
    if (matchFrom != null) {
      const matchTo = assetList.find(cc => cc === toCurrencyCode)
      if (matchTo != null) {
        return true
      }
    }
  }
  return false
}; exports.isLikeKind = isLikeKind

 const getTokenId = (
  coreWallet,
  currencyCode
) => {
  if (coreWallet.currencyInfo.currencyCode === currencyCode) return
  const { allTokens } = coreWallet.currencyConfig
  return Object.keys(allTokens).find(
    edgeToken => allTokens[edgeToken].currencyCode === currencyCode
  )
}; exports.getTokenId = getTokenId

 const consify = (val) =>
  console.log(JSON.stringify(val, null, 2)); exports.consify = consify
