"use strict";Object.defineProperty(exports, "__esModule", {value: true}); function _nullishCoalesce(lhs, rhsFn) { if (lhs != null) { return lhs; } else { return rhsFn(); } } function _optionalChain(ops) { let lastAccessLHS = undefined; let value = ops[0]; let i = 1; while (i < ops.length) { const op = ops[i]; const fn = ops[i + 1]; i += 2; if ((op === 'optionalAccess' || op === 'optionalCall') && value == null) { return undefined; } if (op === 'access' || op === 'optionalAccess') { lastAccessLHS = value; value = fn(value); } else if (op === 'call' || op === 'optionalCall') { value = fn((...args) => value.call(lastAccessLHS, ...args)); lastAccessLHS = undefined; } } return value; }var _biggystring = require('biggystring');







var _cleaners = require('cleaners');










var _types = require('edge-core-js/types');








var _swaphelpers = require('../swap-helpers');
var _utils = require('../util/utils');


const pluginId = 'changehero'

const swapInfo = {
  pluginId,
  isDex: false,
  displayName: 'ChangeHero',
  supportEmail: 'support@changehero.io'
}

const asInitOptions = _cleaners.asObject.call(void 0, {
  apiKey: _cleaners.asString
})

// See https://changehero.io/currencies for list of supported currencies
const INVALID_CURRENCY_CODES = {
  from: {
    optimism: ['VELO']
  },
  to: {
    zcash: ['ZEC'] // ChangeHero doesn't support sending to shielded addresses
  }
}

const orderUri = 'https://changehero.io/transaction/'
const uri = 'https://api.changehero.io/v2'
const expirationFixedMs = 1000 * 60

const asGetFixRateReply = _cleaners.asObject.call(void 0, {
  result: _cleaners.asArray.call(void 0, 
    _cleaners.asObject.call(void 0, {
      id: _cleaners.asString,
      maxFrom: _cleaners.asString,
      maxTo: _cleaners.asString,
      minFrom: _cleaners.asString,
      minTo: _cleaners.asString
      // from: asString,
      // to: asString,
    })
  )
})

const asCreateFixTransactionReply = _cleaners.asObject.call(void 0, {
  result: _cleaners.asObject.call(void 0, {
    id: _cleaners.asString,
    status: _cleaners.asString,
    amountExpectedFrom: _cleaners.asEither.call(void 0, _cleaners.asString, _cleaners.asNumber),
    amountExpectedTo: _cleaners.asEither.call(void 0, _cleaners.asString, _cleaners.asNumber),
    payinAddress: _cleaners.asString,
    payinExtraId: _cleaners.asOptional.call(void 0, _cleaners.asString),
    currencyFrom: _cleaners.asString,
    currencyTo: _cleaners.asString,
    payoutAddress: _cleaners.asString,
    payoutExtraId: _cleaners.asOptional.call(void 0, _cleaners.asString)
  })
})

function checkReply(
  reply,
  request
) {
  if (reply.error != null) {
    if (
      reply.error.code === -32602 ||
      (_nullishCoalesce(_optionalChain([reply, 'access', _ => _.error, 'access', _2 => _2.message, 'optionalAccess', _3 => _3.includes, 'call', _4 => _4('Invalid currency:')]), () => ( false)))
    ) {
      throw new (0, _types.SwapCurrencyError)(
        swapInfo,
        request.fromCurrencyCode,
        request.toCurrencyCode
      )
    }
    throw new Error('ChangeHero error: ' + JSON.stringify(reply.error))
  }
}

 function makeChangeHeroPlugin(
  opts
) {
  const { io } = opts
  const { fetchCors = io.fetch } = io
  const { apiKey } = asInitOptions(opts.initOptions)

  async function call(json) {
    const body = JSON.stringify(json)

    const headers = {
      'Content-Type': 'application/json',
      'api-key': apiKey
    }
    const response = await fetchCors(uri, { method: 'POST', body, headers })

    if (!response.ok) {
      throw new Error(`ChangeHero returned error code ${response.status}`)
    }
    return await response.json()
  }

  async function getFixedQuote(
    request
  ) {
    const [fromAddress, toAddress] = await Promise.all([
      _utils.getAddress.call(void 0, request.fromWallet),
      _utils.getAddress.call(void 0, request.toWallet)
    ])
    const { fromCurrencyCode, toCurrencyCode } = _swaphelpers.getCodes.call(void 0, request)

    // The chain codes are undocumented but ChangeHero uses Edge pluginIds for these values (confirmed via Slack)
    const fromMainnetCode = request.fromWallet.currencyInfo.pluginId
    const toMainnetCode = request.toWallet.currencyInfo.pluginId

    const quoteAmount =
      request.quoteFor === 'from'
        ? await request.fromWallet.nativeToDenomination(
            request.nativeAmount,
            fromCurrencyCode
          )
        : await request.toWallet.nativeToDenomination(
            request.nativeAmount,
            toCurrencyCode
          )

    const fixRate = {
      jsonrpc: '2.0',
      id: 'one',
      method: 'getFixRate',
      params: {
        from: fromCurrencyCode,
        to: toCurrencyCode,
        chainFrom: fromMainnetCode,
        chainTo: toMainnetCode
      }
    }
    const fixedRateQuote = await call(fixRate)

    checkReply(fixedRateQuote, request)

    const [
      { id: responseId, maxFrom, maxTo, minFrom, minTo }
    ] = asGetFixRateReply(fixedRateQuote).result
    const maxFromNative = await request.fromWallet.denominationToNative(
      maxFrom,
      fromCurrencyCode
    )
    const maxToNative = await request.toWallet.denominationToNative(
      maxTo,
      toCurrencyCode
    )
    const minFromNative = await request.fromWallet.denominationToNative(
      minFrom,
      fromCurrencyCode
    )
    const minToNative = await request.toWallet.denominationToNative(
      minTo,
      toCurrencyCode
    )

    if (request.quoteFor === 'from') {
      if (_biggystring.gt.call(void 0, quoteAmount, maxFrom)) {
        throw new (0, _types.SwapAboveLimitError)(swapInfo, maxFromNative)
      }
      if (_biggystring.lt.call(void 0, quoteAmount, minFrom)) {
        throw new (0, _types.SwapBelowLimitError)(swapInfo, minFromNative)
      }
    } else {
      if (_biggystring.gt.call(void 0, quoteAmount, maxTo)) {
        throw new (0, _types.SwapAboveLimitError)(swapInfo, maxToNative, 'to')
      }
      if (_biggystring.lt.call(void 0, quoteAmount, minTo)) {
        throw new (0, _types.SwapBelowLimitError)(swapInfo, minToNative, 'to')
      }
    }

    const params =
      request.quoteFor === 'from'
        ? {
            amount: quoteAmount,
            from: fromCurrencyCode,
            to: toCurrencyCode,
            chainFrom: fromMainnetCode,
            chainTo: toMainnetCode,
            address: toAddress,
            extraId: null,
            refundAddress: fromAddress,
            refundExtraId: null,
            rateId: responseId
          }
        : {
            amountTo: quoteAmount,
            from: fromCurrencyCode,
            to: toCurrencyCode,
            chainFrom: fromMainnetCode,
            chainTo: toMainnetCode,
            address: toAddress,
            extraId: null,
            refundAddress: fromAddress,
            refundExtraId: null,
            rateId: responseId
          }
    const reply = {
      jsonrpc: '2.0',
      id: 2,
      method: 'createFixTransaction',
      params
    }

    const sendReply = await call(reply)

    // NOTE: Testing showed the undocumented `chainFrom` and `chainTo` fields in sendReply are present in the response but are null.
    // Tested with mainnet currency codes in addition to the pluginIds as detailed above.

    checkReply(sendReply, request)

    const quoteInfo = asCreateFixTransactionReply(sendReply).result
    const amountExpectedFromNative = await request.fromWallet.denominationToNative(
      `${quoteInfo.amountExpectedFrom.toString()}`,
      fromCurrencyCode
    )
    const amountExpectedToNative = await request.toWallet.denominationToNative(
      `${quoteInfo.amountExpectedTo.toString()}`,
      toCurrencyCode
    )

    const spendInfo = {
      currencyCode: fromCurrencyCode,
      spendTargets: [
        {
          nativeAmount: amountExpectedFromNative,
          publicAddress: quoteInfo.payinAddress,
          uniqueIdentifier: _nullishCoalesce(quoteInfo.payinExtraId, () => ( undefined))
        }
      ],
      networkFeeOption:
        request.fromCurrencyCode.toUpperCase() === 'BTC' ? 'high' : 'standard',
      swapData: {
        orderUri: orderUri + quoteInfo.id,
        orderId: quoteInfo.id,
        isEstimate: false,
        payoutAddress: toAddress,
        payoutCurrencyCode: toCurrencyCode,
        payoutNativeAmount: amountExpectedToNative,
        payoutWalletId: request.toWallet.id,
        plugin: { ...swapInfo },
        refundAddress: fromAddress
      }
    }

    return {
      request,
      spendInfo,
      swapInfo,
      fromNativeAmount: amountExpectedFromNative,
      expirationDate: new Date(Date.now() + expirationFixedMs)
    }
  }

  const out = {
    swapInfo,
    async fetchSwapQuote(req) {
      const request = _utils.convertRequest.call(void 0, req)
      _swaphelpers.checkInvalidCodes.call(void 0, INVALID_CURRENCY_CODES, request, swapInfo)

      const newRequest = await _swaphelpers.getMaxSwappable.call(void 0, getFixedQuote, request)
      const swapOrder = await getFixedQuote(newRequest)
      return await _swaphelpers.makeSwapPluginQuote.call(void 0, swapOrder)
    }
  }

  return out
} exports.makeChangeHeroPlugin = makeChangeHeroPlugin;
