"use strict";Object.defineProperty(exports, "__esModule", {value: true}); function _nullishCoalesce(lhs, rhsFn) { if (lhs != null) { return lhs; } else { return rhsFn(); } } function _optionalChain(ops) { let lastAccessLHS = undefined; let value = ops[0]; let i = 1; while (i < ops.length) { const op = ops[i]; const fn = ops[i + 1]; i += 2; if ((op === 'optionalAccess' || op === 'optionalCall') && value == null) { return undefined; } if (op === 'access' || op === 'optionalAccess') { lastAccessLHS = value; value = fn(value); } else if (op === 'call' || op === 'optionalCall') { value = fn((...args) => value.call(lastAccessLHS, ...args)); lastAccessLHS = undefined; } } return value; }var _biggystring = require('biggystring');
var _cleaners = require('cleaners');









var _types = require('edge-core-js/types');







var _swaphelpers = require('../../swap-helpers');
var _biggystringplus = require('../../util/biggystringplus');








var _utils = require('../../util/utils');
var _types3 = require('../types');
var _defiUtils = require('./defiUtils');

const pluginId = 'lifi'
const swapInfo = {
  pluginId,
  isDex: true,
  displayName: 'LI.FI',
  supportEmail: 'support@edge.app'
}

const asInitOptions = _cleaners.asObject.call(void 0, {
  affiliateFeeBasis: _cleaners.asOptional.call(void 0, _cleaners.asString, '50'),
  appId: _cleaners.asOptional.call(void 0, _cleaners.asString, 'edge'),
  integrator: _cleaners.asOptional.call(void 0, _cleaners.asString, 'edgeapp')
})

const LIFI_SERVERS_DEFAULT = ['https://li.quest']
const EXPIRATION_MS = 1000 * 60
const EXCHANGE_INFO_UPDATE_FREQ_MS = 60000

 const INVALID_CURRENCY_CODES = {
  from: {},
  to: {
    zcash: ['ZEC']
  }
}; exports.INVALID_CURRENCY_CODES = INVALID_CURRENCY_CODES

// Network names that don't match parent network currency code
// See https://docs.li.fi/list-chains-bridges-dexs#chains
const MAINNET_CODE_TRANSCRIPTION = {
  arbitrum: 'ARB',
  aurora: 'AUR',
  avalanche: 'AVA',
  binancesmartchain: 'BSC',
  celo: 'CEL',
  cronos: 'CRO',
  ethereum: 'ETH',
  evmos: 'EVM',
  fantom: 'FTM',
  fuse: 'FUS',
  gnosis: 'DAI',
  harmony: 'ONE',
  moonbeam: 'MOO',
  moonriver: 'MOR',
  okexchain: 'OKT',
  optimism: 'OPT',
  polygon: 'POL',
  velas: 'VEL'
}

const asExchangeInfo = _cleaners.asObject.call(void 0, {
  swap: _cleaners.asObject.call(void 0, {
    plugins: _cleaners.asObject.call(void 0, {
      lifi: _cleaners.asOptional.call(void 0, 
        _cleaners.asObject.call(void 0, {
          // perAssetSpread: asOptional(asArray(asAssetSpread)),
          // volatilitySpread: asOptional(asNumber),
          // likeKindVolatilitySpread: asOptional(asNumber),
          // daVolatilitySpread: asOptional(asNumber),
          lifiServers: _cleaners.asOptional.call(void 0, _cleaners.asArray.call(void 0, _cleaners.asString))
        })
      )
    })
  })
})

// const asToken = asObject({
//   address: asString, // "0x2791bca1f2de4661ed88a30c99a7a9449aa84174",
//   chainId: asNumber, // 137,
//   symbol: asString, // "USDC",
//   decimals: asNumber, // 6,
//   name: asString, // "USDC",
//   priceUSD: asNumberString, // "1",
//   coinKey: asString // "USDC"
// })

// const asAction = asObject({
//   fromChainId: asNumber,
//   fromAmount: asNumberString,
//   fromToken: asToken,
//   toChainId: asNumber,
//   toToken: asToken
// })

// const asFeeCost = asObject({
//   amount: asNumberString, // "56495962827064236208",
//   token: asToken
// })

const asEstimate = _cleaners.asObject.call(void 0, {
  fromAmount: _types3.asNumberString, // "400000",
  toAmount: _types3.asNumberString, // "237318132569913",
  toAmountMin: _types3.asNumberString, // "225452225941418",
  approvalAddress: _cleaners.asString, // "0x1231DEB6f5749EF6cE6943a275A1D3E7486F4EaE",
  executionDuration: _cleaners.asNumber // 1168,
  // feeCosts: asArray(asFeeCost)
})

const asTransactionRequest = _cleaners.asObject.call(void 0, {
  data: _cleaners.asString,
  to: _cleaners.asString, // '0x1231DEB6f5749EF6cE6943a275A1D3E7486F4EaE'
  value: _cleaners.asString, // '0x00'
  from: _cleaners.asString, // '0x0b0901e9cef9eaed5753519177e3c7cfd0ef96ef'
  chainId: _cleaners.asNumber, // 1
  gasPrice: _cleaners.asString, // '0x03aca2109d'
  gasLimit: _cleaners.asString // '0x08a3df'
})

const asIncludedStep = _cleaners.asObject.call(void 0, {
  toolDetails: _cleaners.asObject.call(void 0, {
    name: _cleaners.asString
  })
})

const asV1Quote = _cleaners.asObject.call(void 0, {
  id: _cleaners.asString,
  type: _cleaners.asString,
  estimate: asEstimate,
  includedSteps: _cleaners.asArray.call(void 0, asIncludedStep),
  // action: asAction,
  transactionRequest: asTransactionRequest
})



let exchangeInfo
let exchangeInfoLastUpdate = 0

 function makeLifiPlugin(opts) {
  const { io, log } = opts
  const { affiliateFeeBasis } = asInitOptions(opts.initOptions)
  const affiliateFee = _biggystringplus.div18.call(void 0, affiliateFeeBasis, '10000')

  const headers = {
    'Content-Type': 'application/json'
  }

  const fetchSwapQuoteInner = async (
    request
  ) => {
    const {
      fromCurrencyCode,
      fromTokenId,
      toCurrencyCode,
      toTokenId,
      nativeAmount,
      fromWallet,
      toWallet,
      quoteFor
    } = request
    if (quoteFor !== 'from') {
      throw new (0, _types.SwapCurrencyError)(swapInfo, fromCurrencyCode, toCurrencyCode)
    }

    const fromToken = fromWallet.currencyConfig.allTokens[_nullishCoalesce(fromTokenId, () => ( ''))]
    let fromContractAddress
    let sendingToken = false
    if (fromCurrencyCode === fromWallet.currencyInfo.currencyCode) {
      fromContractAddress = fromCurrencyCode
    } else {
      sendingToken = true
      fromContractAddress = _optionalChain([fromToken, 'optionalAccess', _ => _.networkLocation, 'optionalAccess', _2 => _2.contractAddress])
    }

    const toToken = toWallet.currencyConfig.allTokens[_nullishCoalesce(toTokenId, () => ( ''))]
    let toContractAddress
    if (toCurrencyCode === toWallet.currencyInfo.currencyCode) {
      toContractAddress = toCurrencyCode
    } else {
      toContractAddress = _optionalChain([toToken, 'optionalAccess', _3 => _3.networkLocation, 'optionalAccess', _4 => _4.contractAddress])
    }

    if (fromContractAddress == null || toContractAddress == null) {
      throw new (0, _types.SwapCurrencyError)(swapInfo, fromCurrencyCode, toCurrencyCode)
    }

    const { appId, integrator } = asInitOptions(opts.initOptions)
    const { fetchCors = io.fetch } = io

    // Do not support transfer between same assets
    if (
      fromWallet.currencyInfo.pluginId === toWallet.currencyInfo.pluginId &&
      request.fromCurrencyCode === request.toCurrencyCode
    ) {
      throw new (0, _types.SwapCurrencyError)(swapInfo, fromCurrencyCode, toCurrencyCode)
    }

    let lifiServers = LIFI_SERVERS_DEFAULT

    _swaphelpers.checkInvalidCodes.call(void 0, exports.INVALID_CURRENCY_CODES, request, swapInfo)

    // Grab addresses:
    const fromAddress = await _utils.getAddress.call(void 0, fromWallet)
    const toAddress = await _utils.getAddress.call(void 0, toWallet)

    const fromMainnetCode =
      MAINNET_CODE_TRANSCRIPTION[fromWallet.currencyInfo.pluginId]
    const toMainnetCode =
      MAINNET_CODE_TRANSCRIPTION[toWallet.currencyInfo.pluginId]

    if (fromMainnetCode == null || toMainnetCode == null) {
      throw new (0, _types.SwapCurrencyError)(swapInfo, fromCurrencyCode, toCurrencyCode)
    }

    const now = Date.now()
    if (
      now - exchangeInfoLastUpdate > EXCHANGE_INFO_UPDATE_FREQ_MS ||
      exchangeInfo == null
    ) {
      try {
        const exchangeInfoResponse = await _utils.promiseWithTimeout.call(void 0, 
          _utils.fetchInfo.call(void 0, fetchCors, `v1/exchangeInfo/${appId}`)
        )

        if (exchangeInfoResponse.ok === true) {
          exchangeInfo = asExchangeInfo(await exchangeInfoResponse.json())
          exchangeInfoLastUpdate = now
        } else {
          // Error is ok. We just use defaults
          const text = await exchangeInfoResponse.text()
          log.warn(
            `Error getting info server exchangeInfo. Using defaults... Error: ${text}`
          )
        }
      } catch (e) {
        log.warn(
          'Error getting info server exchangeInfo. Using defaults...',
          e.message
        )
      }
    }

    if (exchangeInfo != null) {
      const { lifi } = exchangeInfo.swap.plugins
      lifiServers = _nullishCoalesce(_optionalChain([lifi, 'optionalAccess', _5 => _5.lifiServers]), () => ( lifiServers))
    }

    const params = _utils.makeQueryParams.call(void 0, {
      fromChain: fromMainnetCode,
      toChain: toMainnetCode,
      fromToken: fromContractAddress,
      toToken: toContractAddress,
      fromAmount: nativeAmount,
      fromAddress,
      toAddress,
      integrator,
      fee: affiliateFee
    })
    // Get current pool
    const [quoteResponse] = await Promise.all([
      _utils.fetchWaterfall.call(void 0, fetchCors, lifiServers, `v1/quote?${params}`, {
        headers
      })
    ])

    if (!quoteResponse.ok) {
      const responseText = await quoteResponse.text()
      throw new Error(`Lifi could not fetch v1/quote: ${responseText}`)
    }

    const quoteJson = await quoteResponse.json()
    const quote = asV1Quote(quoteJson)
    const { estimate, includedSteps, transactionRequest } = quote
    const { approvalAddress, toAmountMin } = estimate

    const { data, gasLimit, gasPrice } = transactionRequest
    const gasPriceDecimal = _utils.hexToDecimal.call(void 0, gasPrice)
    const gasPriceGwei = _biggystringplus.div18.call(void 0, gasPriceDecimal, '1000000000')
    const providers = includedSteps.map(s => s.toolDetails.name)

    let preTx
    if (sendingToken) {
      const approvalData = await _defiUtils.getEvmApprovalData.call(void 0, {
        contractAddress: approvalAddress,
        assetAddress: fromContractAddress,
        nativeAmount
      })

      const spendInfo = {
        currencyCode: request.fromCurrencyCode,
        spendTargets: [
          {
            memo: approvalData,
            nativeAmount: '0',
            publicAddress: fromContractAddress
          }
        ],
        networkFeeOption: 'custom',
        customNetworkFee: {
          gasPrice: gasPriceGwei
        },
        metadata: {
          name: 'Li.Fi',
          category: 'expense:Token Approval'
        }
      }
      preTx = await request.fromWallet.makeSpend(spendInfo)
    }

    const spendInfo = {
      currencyCode: request.fromCurrencyCode,
      spendTargets: [
        {
          memo: data,
          nativeAmount: _biggystring.mul.call(void 0, transactionRequest.value, '1'),
          publicAddress: approvalAddress
        }
      ],
      networkFeeOption: 'custom',
      customNetworkFee: {
        // XXX Hack. Lifi doesn't properly estimate ethereum gas limits. Increase by 40%
        gasLimit: _biggystring.round.call(void 0, _biggystring.mul.call(void 0, _utils.hexToDecimal.call(void 0, gasLimit), '1.4'), 0),
        gasPrice: gasPriceGwei
      },
      swapData: {
        isEstimate: false,
        payoutAddress: toAddress,
        payoutCurrencyCode: toCurrencyCode,
        payoutNativeAmount: toAmountMin,
        payoutWalletId: toWallet.id,
        plugin: { ...swapInfo }
      }
    }

    const providersStr = providers.join(' -> ')
    const metadataNotes = `DEX Providers: ${providersStr}`

    return {
      request,
      spendInfo,
      swapInfo,
      fromNativeAmount: nativeAmount,
      expirationDate: new Date(Date.now() + EXPIRATION_MS),
      preTx,
      metadataNotes
    }
  }

  const out = {
    swapInfo,

    async fetchSwapQuote(req) {
      const request = _utils.convertRequest.call(void 0, req)

      const newRequest = await _swaphelpers.getMaxSwappable.call(void 0, 
        async r => await fetchSwapQuoteInner(r),
        request
      )
      const swapOrder = await fetchSwapQuoteInner(newRequest)
      return await _swaphelpers.makeSwapPluginQuote.call(void 0, swapOrder)
    }
  }
  return out
} exports.makeLifiPlugin = makeLifiPlugin;
