"use strict";Object.defineProperty(exports, "__esModule", {value: true}); function _nullishCoalesce(lhs, rhsFn) { if (lhs != null) { return lhs; } else { return rhsFn(); } }var _biggystring = require('biggystring');







var _cleaners = require('cleaners');













var _types = require('edge-core-js/types');








var _swaphelpers = require('../../swap-helpers');








var _utils = require('../../util/utils');

var _defiUtils = require('./defiUtils');

const pluginId = 'thorchain'
const swapInfo = {
  pluginId,
  isDex: true,
  displayName: 'Thorchain',
  supportEmail: 'support@edge.app'
}

 const MIDGARD_SERVERS_DEFAULT = ['https://midgard.thorchain.info']; exports.MIDGARD_SERVERS_DEFAULT = MIDGARD_SERVERS_DEFAULT
 const THORNODE_SERVERS_DEFAULT = ['https://thornode.ninerealms.com']; exports.THORNODE_SERVERS_DEFAULT = THORNODE_SERVERS_DEFAULT
 const EXPIRATION_MS = 1000 * 60; exports.EXPIRATION_MS = EXPIRATION_MS
 const DIVIDE_PRECISION = 16; exports.DIVIDE_PRECISION = DIVIDE_PRECISION
 const EXCHANGE_INFO_UPDATE_FREQ_MS = 60000; exports.EXCHANGE_INFO_UPDATE_FREQ_MS = EXCHANGE_INFO_UPDATE_FREQ_MS
 const EVM_SEND_GAS = '80000'; exports.EVM_SEND_GAS = EVM_SEND_GAS
 const EVM_TOKEN_SEND_GAS = '80000'; exports.EVM_TOKEN_SEND_GAS = EVM_TOKEN_SEND_GAS
 const MIN_USD_SWAP = '10'; exports.MIN_USD_SWAP = MIN_USD_SWAP
 const THOR_LIMIT_UNITS = '100000000'; exports.THOR_LIMIT_UNITS = THOR_LIMIT_UNITS
const AFFILIATE_FEE_BASIS_DEFAULT = '50'
const STREAMING_INTERVAL_DEFAULT = 10
const STREAMING_QUANTITY_DEFAULT = 10
const STREAMING_INTERVAL_NOSTREAM = 1
const STREAMING_QUANTITY_NOSTREAM = 1

// ----------------------------------------------------------------------------
// Volatility spread logic
//
// BTC/BCH have 10 min block times which can lead to more volatility
// so set them at the highest volatility of 1.5%
//
// LTC/DOGE have ~2min block time creating medium volatility so set to 1%
// Remaining chains are faster EVM chains so use the default 0.75% for most
// assets and 0.5% for like-kind assets
//
// These are all defaults and can be changed via the info server
// ----------------------------------------------------------------------------
 const VOLATILITY_SPREAD_DEFAULT = 0.0075; exports.VOLATILITY_SPREAD_DEFAULT = VOLATILITY_SPREAD_DEFAULT
 const LIKE_KIND_VOLATILITY_SPREAD_DEFAULT = 0.005; exports.LIKE_KIND_VOLATILITY_SPREAD_DEFAULT = LIKE_KIND_VOLATILITY_SPREAD_DEFAULT
 const VOLATILITY_SPREAD_STREAMING_DEFAULT = 0.001; exports.VOLATILITY_SPREAD_STREAMING_DEFAULT = VOLATILITY_SPREAD_STREAMING_DEFAULT
 const LIKE_KIND_VOLATILITY_SPREAD_STREAMING_DEFAULT = 0; exports.LIKE_KIND_VOLATILITY_SPREAD_STREAMING_DEFAULT = LIKE_KIND_VOLATILITY_SPREAD_STREAMING_DEFAULT
 const PER_ASSET_SPREAD_DEFAULT = [
  {
    sourcePluginId: 'bitcoin',
    volatilitySpread: 0.015,
    sourceTokenId: undefined,
    sourceCurrencyCode: undefined,
    destPluginId: undefined,
    destTokenId: undefined,
    destCurrencyCode: undefined
  },
  {
    sourcePluginId: 'bitcoincash',
    volatilitySpread: 0.015,
    sourceTokenId: undefined,
    sourceCurrencyCode: undefined,
    destPluginId: undefined,
    destTokenId: undefined,
    destCurrencyCode: undefined
  },
  {
    sourcePluginId: 'dogecoin',
    volatilitySpread: 0.01,
    sourceTokenId: undefined,
    sourceCurrencyCode: undefined,
    destPluginId: undefined,
    destTokenId: undefined,
    destCurrencyCode: undefined
  },
  {
    sourcePluginId: 'litecoin',
    volatilitySpread: 0.01,
    sourceTokenId: undefined,
    sourceCurrencyCode: undefined,
    destPluginId: undefined,
    destTokenId: undefined,
    destCurrencyCode: undefined
  }
]; exports.PER_ASSET_SPREAD_DEFAULT = PER_ASSET_SPREAD_DEFAULT

 const INVALID_CURRENCY_CODES = {
  from: {
    optimism: ['VELO']
  },
  to: {
    zcash: ['ZEC']
  }
}; exports.INVALID_CURRENCY_CODES = INVALID_CURRENCY_CODES

 const EVM_CURRENCY_CODES = {
  AVAX: true,
  BCH: false,
  BNB: false,
  BTC: false,
  DOGE: false,
  ETC: true,
  ETH: true,
  FTM: true,
  LTC: false,
  THOR: false
}; exports.EVM_CURRENCY_CODES = EVM_CURRENCY_CODES

// Network names that don't match parent network currency code
 const MAINNET_CODE_TRANSCRIPTION = {
  avalanche: 'AVAX',
  binancechain: 'BNB',
  bitcoin: 'BTC',
  bitcoincash: 'BCH',
  dogecoin: 'DOGE',
  ethereum: 'ETH',
  litecoin: 'LTC',
  thorchain: 'THOR'
}; exports.MAINNET_CODE_TRANSCRIPTION = MAINNET_CODE_TRANSCRIPTION

 const asInitOptions = _cleaners.asObject.call(void 0, {
  appId: _cleaners.asOptional.call(void 0, _cleaners.asString, 'edge'),
  affiliateFeeBasis: _cleaners.asOptional.call(void 0, _cleaners.asString, AFFILIATE_FEE_BASIS_DEFAULT),
  ninerealmsClientId: _cleaners.asOptional.call(void 0, _cleaners.asString, ''),
  thorname: _cleaners.asOptional.call(void 0, _cleaners.asString, 'ej')
}); exports.asInitOptions = asInitOptions

const asMinAmount = _cleaners.asObject.call(void 0, {
  minInputAmount: _cleaners.asString
})

 const asInboundAddresses = _cleaners.asArray.call(void 0, 
  _cleaners.asObject.call(void 0, {
    address: _cleaners.asString,
    chain: _cleaners.asString,
    outbound_fee: _cleaners.asString,
    halted: _cleaners.asBoolean,
    pub_key: _cleaners.asString,
    router: _cleaners.asOptional.call(void 0, _cleaners.asString)
  })
); exports.asInboundAddresses = asInboundAddresses

 const asPool = _cleaners.asObject.call(void 0, {
  asset: _cleaners.asString,
  status: _cleaners.asString,
  assetPrice: _cleaners.asString,
  assetPriceUSD: _cleaners.asString,
  assetDepth: _cleaners.asString,
  runeDepth: _cleaners.asString
}); exports.asPool = asPool

 const asAssetSpread = _cleaners.asObject.call(void 0, {
  sourcePluginId: _cleaners.asOptional.call(void 0, _cleaners.asString),
  sourceTokenId: _cleaners.asOptional.call(void 0, _cleaners.asString),
  sourceCurrencyCode: _cleaners.asOptional.call(void 0, _cleaners.asString),
  destPluginId: _cleaners.asOptional.call(void 0, _cleaners.asString),
  destTokenId: _cleaners.asOptional.call(void 0, _cleaners.asString),
  destCurrencyCode: _cleaners.asOptional.call(void 0, _cleaners.asString),
  volatilitySpread: _cleaners.asNumber
}); exports.asAssetSpread = asAssetSpread

 const asExchangeInfo = _cleaners.asObject.call(void 0, {
  swap: _cleaners.asObject.call(void 0, {
    plugins: _cleaners.asObject.call(void 0, {
      thorchain: _cleaners.asObject.call(void 0, {
        perAssetSpread: _cleaners.asArray.call(void 0, exports.asAssetSpread),
        perAssetSpreadStreaming: _cleaners.asOptional.call(void 0, _cleaners.asArray.call(void 0, exports.asAssetSpread)),
        volatilitySpread: _cleaners.asNumber,
        volatilitySpreadStreaming: _cleaners.asOptional.call(void 0, _cleaners.asNumber),
        likeKindVolatilitySpread: _cleaners.asNumber,
        likeKindVolatilitySpreadStreaming: _cleaners.asOptional.call(void 0, _cleaners.asNumber),
        daVolatilitySpread: _cleaners.asNumber,
        midgardServers: _cleaners.asArray.call(void 0, _cleaners.asString),
        affiliateFeeBasis: _cleaners.asOptional.call(void 0, _cleaners.asString),
        nineRealmsServers: _cleaners.asOptional.call(void 0, _cleaners.asArray.call(void 0, _cleaners.asString)),
        streamingInterval: _cleaners.asOptional.call(void 0, _cleaners.asNumber),
        streamingQuantity: _cleaners.asOptional.call(void 0, _cleaners.asNumber),
        thornodeServers: _cleaners.asOptional.call(void 0, _cleaners.asArray.call(void 0, _cleaners.asString)),
        thorSwapServers: _cleaners.asOptional.call(void 0, _cleaners.asArray.call(void 0, _cleaners.asString))
      })
    })
  })
}); exports.asExchangeInfo = asExchangeInfo

const asPools = _cleaners.asArray.call(void 0, exports.asPool)

const asQuoteSwap = _cleaners.asObject.call(void 0, {
  // expected_amount_out: asString, // "61409897"
  expected_amount_out_streaming: _cleaners.asString, // "62487221"
  expiry: _cleaners.asNumber, // 1692149478
  // fees: asObject({
  //   affiliate: asString, // "0"
  //   asset: asString, // "ETH.WBTC-0X2260FAC5E5542A773AA44FBCFEDF7C193BC2C599",
  //   outbound: asString // "22117"
  // }),
  inbound_address: _cleaners.asString, // "0x88e8def37dc9d2acd67f1c1574ad09ca49827374",
  // max_streaming_quantity: asNumber, // 18,
  memo: _cleaners.asString, // "=:ETH.WBTC-0X2260FAC5E5542A773AA44FBCFEDF7C193BC2C599:0x04c5998ded94f89263370444ce64a99b7dbc9f46:0/10/0",
  // outbound_delay_blocks: asNumber, // 114,
  // outbound_delay_seconds: asNumber, // 684,
  // recommended_min_amount_in: asString, // "1440032",
  router: _cleaners.asOptional.call(void 0, _cleaners.asString), // "0xD37BbE5744D730a1d98d8DC97c42F0Ca46aD7146",
  // slippage_bps: asNumber, // 92,
  // streaming_slippage_bps: asNumber, // 5,
  streaming_swap_blocks: _cleaners.asNumber, // 170,
  total_swap_seconds: _cleaners.asOptional.call(void 0, _cleaners.asNumber) // 1020,
})









































let exchangeInfo
let exchangeInfoLastUpdate = 0

 function makeThorchainPlugin(
  opts
) {
  const { io, log } = opts
  const { fetchCors = io.fetch } = io
  const initOptions = exports.asInitOptions.call(void 0, opts.initOptions)
  const { appId, thorname, ninerealmsClientId } = initOptions
  let { affiliateFeeBasis = AFFILIATE_FEE_BASIS_DEFAULT } = initOptions

  const headers = {
    'Content-Type': 'application/json',
    'x-client-id': ninerealmsClientId
  }

  const fetchSwapQuoteInner = async (
    request
  ) => {
    const {
      fromCurrencyCode,
      fromTokenId,
      toCurrencyCode,
      toTokenId,
      nativeAmount,
      fromWallet,
      toWallet,
      quoteFor
    } = request
    // Do not support transfer between same assets
    if (
      fromWallet.currencyInfo.pluginId === toWallet.currencyInfo.pluginId &&
      request.fromCurrencyCode === request.toCurrencyCode
    ) {
      throw new (0, _types.SwapCurrencyError)(swapInfo, fromCurrencyCode, toCurrencyCode)
    }

    let midgardServers = exports.MIDGARD_SERVERS_DEFAULT
    let thornodeServers = exports.THORNODE_SERVERS_DEFAULT
    let likeKindVolatilitySpread = exports.LIKE_KIND_VOLATILITY_SPREAD_DEFAULT
    let likeKindVolatilitySpreadStreaming = exports.LIKE_KIND_VOLATILITY_SPREAD_STREAMING_DEFAULT
    let volatilitySpread = exports.VOLATILITY_SPREAD_DEFAULT
    let volatilitySpreadStreaming = exports.VOLATILITY_SPREAD_STREAMING_DEFAULT
    let perAssetSpread = exports.PER_ASSET_SPREAD_DEFAULT
    let perAssetSpreadStreaming = exports.PER_ASSET_SPREAD_DEFAULT
    let streamingInterval = STREAMING_INTERVAL_DEFAULT
    let streamingQuantity = STREAMING_QUANTITY_DEFAULT

    _swaphelpers.checkInvalidCodes.call(void 0, exports.INVALID_CURRENCY_CODES, request, swapInfo)

    // Grab addresses:
    const toAddress = await _utils.getAddress.call(void 0, toWallet)

    const fromMainnetCode =
      exports.MAINNET_CODE_TRANSCRIPTION[fromWallet.currencyInfo.pluginId]
    const toMainnetCode =
      exports.MAINNET_CODE_TRANSCRIPTION[toWallet.currencyInfo.pluginId]

    if (fromMainnetCode == null || toMainnetCode == null) {
      throw new (0, _types.SwapCurrencyError)(swapInfo, fromCurrencyCode, toCurrencyCode)
    }

    const now = Date.now()
    if (
      now - exchangeInfoLastUpdate > exports.EXCHANGE_INFO_UPDATE_FREQ_MS ||
      exchangeInfo == null
    ) {
      try {
        const exchangeInfoResponse = await _utils.promiseWithTimeout.call(void 0, 
          _utils.fetchInfo.call(void 0, fetchCors, `v1/exchangeInfo/${appId}`)
        )

        if (exchangeInfoResponse.ok === true) {
          exchangeInfo = exports.asExchangeInfo.call(void 0, await exchangeInfoResponse.json())
          exchangeInfoLastUpdate = now
        } else {
          // Error is ok. We just use defaults
          log('Error getting info server exchangeInfo. Using defaults...')
        }
      } catch (e) {
        log(
          'Error getting info server exchangeInfo. Using defaults...',
          e.message
        )
      }
    }

    if (exchangeInfo != null) {
      const { thorchain } = exchangeInfo.swap.plugins
      likeKindVolatilitySpread =
        exchangeInfo.swap.plugins.thorchain.likeKindVolatilitySpread
      volatilitySpread = thorchain.volatilitySpread
      likeKindVolatilitySpreadStreaming =
        _nullishCoalesce(exchangeInfo.swap.plugins.thorchain.likeKindVolatilitySpreadStreaming, () => (
        likeKindVolatilitySpreadStreaming))
      volatilitySpreadStreaming =
        _nullishCoalesce(thorchain.volatilitySpreadStreaming, () => ( volatilitySpreadStreaming))
      midgardServers = thorchain.midgardServers
      thornodeServers = _nullishCoalesce(thorchain.thornodeServers, () => ( thornodeServers))
      perAssetSpread = thorchain.perAssetSpread
      perAssetSpreadStreaming =
        _nullishCoalesce(thorchain.perAssetSpreadStreaming, () => ( perAssetSpreadStreaming))
      affiliateFeeBasis = _nullishCoalesce(thorchain.affiliateFeeBasis, () => ( affiliateFeeBasis))
      streamingInterval = _nullishCoalesce(thorchain.streamingInterval, () => ( streamingInterval))
      streamingQuantity = _nullishCoalesce(thorchain.streamingQuantity, () => ( streamingQuantity))
    }

    const volatilitySpreadFinal = exports.getVolatilitySpread.call(void 0, {
      fromPluginId: fromWallet.currencyInfo.pluginId,
      fromTokenId,
      fromCurrencyCode,
      toPluginId: toWallet.currencyInfo.pluginId,
      toTokenId,
      toCurrencyCode,
      likeKindVolatilitySpread,
      volatilitySpread,
      perAssetSpread
    })

    const volatilitySpreadStreamingFinal = exports.getVolatilitySpread.call(void 0, {
      fromPluginId: fromWallet.currencyInfo.pluginId,
      fromTokenId,
      fromCurrencyCode,
      toPluginId: toWallet.currencyInfo.pluginId,
      toTokenId,
      toCurrencyCode,
      likeKindVolatilitySpread: likeKindVolatilitySpreadStreaming,
      volatilitySpread: volatilitySpreadStreaming,
      perAssetSpread: perAssetSpreadStreaming
    })

    log.warn(`volatilitySpreadFinal: ${volatilitySpreadFinal.toString()}`)
    log.warn(
      `volatilitySpreadStreamingFinal: ${volatilitySpreadStreamingFinal.toString()}`
    )

    // Get current pool
    const poolResponse = await _utils.fetchWaterfall.call(void 0, 
      fetchCors,
      midgardServers,
      'v2/pools',
      { headers }
    )

    if (!poolResponse.ok) {
      const responseText = await poolResponse.text()
      throw new Error(`Thorchain could not fetch pools: ${responseText}`)
    }

    // Nine realms servers removed minAmount support so disable for now but keep all code
    // logic so we can easily enable in the future with new API
    const minAmount = undefined

    const poolJson = await poolResponse.json()
    const pools = asPools(poolJson)

    const sourcePool = pools.find(pool => {
      const [asset] = pool.asset.split('-')
      return asset === `${fromMainnetCode}.${fromCurrencyCode}`
    })
    if (sourcePool == null) {
      throw new (0, _types.SwapCurrencyError)(swapInfo, fromCurrencyCode, toCurrencyCode)
    }
    const [
      sourceAsset,
      sourceTokenContractAddressAllCaps
    ] = sourcePool.asset.split('-')
    const sourceTokenContractAddress =
      sourceTokenContractAddressAllCaps != null
        ? sourceTokenContractAddressAllCaps.toLowerCase()
        : undefined
    log(`sourceAsset: ${sourceAsset}`)

    const destPool = pools.find(pool => {
      const [asset] = pool.asset.split('-')
      return asset === `${toMainnetCode}.${toCurrencyCode}`
    })
    if (destPool == null) {
      throw new (0, _types.SwapCurrencyError)(swapInfo, fromCurrencyCode, toCurrencyCode)
    }

    let calcResponse
    if (quoteFor === 'from') {
      calcResponse = await calcSwapFrom({
        log,
        fetch: fetchCors,
        thornodes: thornodeServers,
        thornodesFetchOptions: { headers },
        fromWallet,
        fromCurrencyCode,
        toWallet,
        toCurrencyCode,
        toAddress,
        nativeAmount,
        minAmount,
        sourcePool,
        destPool,
        thorname,
        volatilitySpreadFinal,
        volatilitySpreadStreamingFinal,
        affiliateFeeBasis,
        streamingInterval,
        streamingQuantity
      })
    } else {
      calcResponse = await calcSwapTo({
        log,
        fetch: fetchCors,
        thornodes: thornodeServers,
        thornodesFetchOptions: { headers },
        fromWallet,
        fromCurrencyCode,
        toWallet,
        toCurrencyCode,
        toAddress,
        nativeAmount,
        minAmount,
        sourcePool,
        destPool,
        thorname,
        volatilitySpreadFinal,
        volatilitySpreadStreamingFinal,
        affiliateFeeBasis,
        streamingInterval,
        streamingQuantity
      })
    }
    const {
      canBePartial,
      fromNativeAmount,
      maxFulfillmentSeconds,
      toNativeAmount,
      router,
      thorAddress
    } = calcResponse
    let { memo } = calcResponse

    let ethNativeAmount = fromNativeAmount
    let publicAddress = thorAddress
    let approvalData
    if (exports.EVM_CURRENCY_CODES[fromMainnetCode]) {
      if (fromMainnetCode !== fromCurrencyCode) {
        if (router == null)
          throw new Error(`Missing router address for ${fromMainnetCode}`)
        if (sourceTokenContractAddress == null)
          throw new Error(
            `Missing sourceTokenContractAddress for ${fromMainnetCode}`
          )
        // Need to use ethers.js to craft a proper tx that calls Thorchain contract, then extract the data payload
        memo = await _defiUtils.getEvmTokenData.call(void 0, {
          assetAddress: sourceTokenContractAddress,
          amountToSwapWei: Number(fromNativeAmount),
          contractAddress: router,
          vaultAddress: thorAddress,
          memo
        })

        // Token transactions send no ETH (or other EVM mainnet coin)
        ethNativeAmount = '0'
        publicAddress = router

        // Check if token approval is required and return necessary data field
        approvalData = await _defiUtils.getEvmApprovalData.call(void 0, {
          contractAddress: router,
          assetAddress: sourceTokenContractAddress,
          nativeAmount: fromNativeAmount
        })
      } else {
        memo = '0x' + Buffer.from(memo).toString('hex')
      }
    } else {
      // Cannot yet do tokens on non-EVM chains
      if (fromMainnetCode !== fromCurrencyCode) {
        throw new (0, _types.SwapCurrencyError)(swapInfo, fromCurrencyCode, toCurrencyCode)
      }
    }

    let preTx
    if (approvalData != null) {
      const spendInfo = {
        currencyCode: request.fromCurrencyCode,
        spendTargets: [
          {
            memo: approvalData,
            nativeAmount: '0',
            publicAddress: sourceTokenContractAddress
          }
        ],
        metadata: {
          name: 'Thorchain',
          category: 'expense:Token Approval'
        }
      }
      preTx = await request.fromWallet.makeSpend(spendInfo)
    }

    const spendInfo = {
      currencyCode: request.fromCurrencyCode,
      spendTargets: [
        {
          memo,
          nativeAmount: ethNativeAmount,
          publicAddress
        }
      ],

      swapData: {
        isEstimate: false,
        payoutAddress: toAddress,
        payoutCurrencyCode: toCurrencyCode,
        payoutNativeAmount: toNativeAmount,
        payoutWalletId: toWallet.id,
        plugin: { ...swapInfo }
      },
      otherParams: {
        outputSort: 'targets'
      }
    }

    if (exports.EVM_CURRENCY_CODES[fromMainnetCode]) {
      if (fromMainnetCode === fromCurrencyCode) {
        // For mainnet coins of EVM chains, use gasLimit override since makeSpend doesn't
        // know how to estimate an ETH spend with extra data
        const gasLimit = exports.getGasLimit.call(void 0, fromMainnetCode, fromCurrencyCode)
        if (gasLimit != null) {
          spendInfo.customNetworkFee = {
            ...spendInfo.customNetworkFee,
            gasLimit
          }
        }
      }
    }

    return {
      canBePartial,
      maxFulfillmentSeconds,
      request,
      spendInfo,
      swapInfo,
      fromNativeAmount,
      expirationDate: new Date(Date.now() + exports.EXPIRATION_MS),
      preTx
    }
  }

  const out = {
    swapInfo,

    async fetchSwapQuote(req) {
      const request = _utils.convertRequest.call(void 0, req)

      const newRequest = await _swaphelpers.getMaxSwappable.call(void 0, fetchSwapQuoteInner, request)
      const swapOrder = await fetchSwapQuoteInner(newRequest)
      return await _swaphelpers.makeSwapPluginQuote.call(void 0, swapOrder)
    }
  }
  return out
} exports.makeThorchainPlugin = makeThorchainPlugin;

const calcSwapFrom = async ({
  log,
  fetch,
  thornodes,
  thornodesFetchOptions,
  fromWallet,
  fromCurrencyCode,
  toWallet,
  toCurrencyCode,
  toAddress,
  nativeAmount,
  minAmount,
  sourcePool,
  destPool,
  thorname,
  volatilitySpreadFinal,
  volatilitySpreadStreamingFinal,
  affiliateFeeBasis,
  streamingInterval,
  streamingQuantity,
  dontCheckLimits = false
}) => {
  const fromNativeAmount = nativeAmount

  // Get exchange rate from source to destination asset
  const fromExchangeAmount = await fromWallet.nativeToDenomination(
    nativeAmount,
    fromCurrencyCode
  )

  log(`fromExchangeAmount: ${fromExchangeAmount}`)

  // Check minimums if we can
  if (!dontCheckLimits) {
    const srcInUsd = _biggystring.mul.call(void 0, sourcePool.assetPriceUSD, fromExchangeAmount)
    let fromMinNativeAmount
    if (_biggystring.lt.call(void 0, srcInUsd, exports.MIN_USD_SWAP)) {
      const minExchangeAmount = _biggystring.div.call(void 0, 
        exports.MIN_USD_SWAP,
        sourcePool.assetPriceUSD,
        exports.DIVIDE_PRECISION
      )
      fromMinNativeAmount = await fromWallet.denominationToNative(
        minExchangeAmount,
        fromCurrencyCode
      )
    }

    if (minAmount != null && _biggystring.lt.call(void 0, fromExchangeAmount, minAmount.minInputAmount)) {
      const tempNativeMin = await fromWallet.denominationToNative(
        minAmount.minInputAmount,
        fromCurrencyCode
      )
      if (_biggystring.gt.call(void 0, tempNativeMin, _nullishCoalesce(fromMinNativeAmount, () => ( '0')))) {
        fromMinNativeAmount = tempNativeMin
      }
    }
    if (fromMinNativeAmount != null) {
      throw new (0, _types.SwapBelowLimitError)(swapInfo, fromMinNativeAmount, 'from')
    }
  }

  const fromThorAmount = _biggystring.mul.call(void 0, fromExchangeAmount, exports.THOR_LIMIT_UNITS)

  const noStreamParams = {
    amount: fromThorAmount,
    from_asset: sourcePool.asset,
    to_asset: destPool.asset,
    destination: toAddress,
    affiliate: thorname,
    affiliate_bps: affiliateFeeBasis,
    streaming_interval: STREAMING_INTERVAL_NOSTREAM,
    streaming_quantity: STREAMING_QUANTITY_NOSTREAM
  }

  const streamParams = {
    ...noStreamParams,
    streaming_interval: streamingInterval,
    streaming_quantity: streamingQuantity
  }
  const bestQuote = await getBestQuote(
    [noStreamParams, streamParams],
    fetch,
    thornodes,
    thornodesFetchOptions
  )

  const {
    expected_amount_out_streaming: toThorAmount,
    inbound_address: thorAddress,
    memo: preMemo,
    router,
    streaming_swap_blocks: streamingSwapBlocks,
    total_swap_seconds: maxFulfillmentSeconds
  } = bestQuote

  const canBePartial = streamingSwapBlocks > 1
  let toThorAmountWithSpread
  if (canBePartial) {
    log(`volatilitySpreadStreamingFinal: ${volatilitySpreadStreamingFinal}`)
    toThorAmountWithSpread = _biggystring.round.call(void 0, 
      _biggystring.mul.call(void 0, _biggystring.sub.call(void 0, '1', volatilitySpreadStreamingFinal), toThorAmount),
      0
    )
  } else {
    log(`volatilitySpreadFinal: ${volatilitySpreadFinal}`)
    toThorAmountWithSpread = _biggystring.round.call(void 0, 
      _biggystring.mul.call(void 0, _biggystring.sub.call(void 0, '1', volatilitySpreadFinal), toThorAmount),
      0
    )
  }

  log(`toThorAmountWithSpread = limit: ${toThorAmountWithSpread}`)

  const toExchangeAmount = _biggystring.div.call(void 0, 
    toThorAmountWithSpread,
    exports.THOR_LIMIT_UNITS,
    exports.DIVIDE_PRECISION
  )
  log(`toExchangeAmount: ${toExchangeAmount}`)

  const toNativeAmountFloat = await toWallet.denominationToNative(
    toExchangeAmount,
    toCurrencyCode
  )
  const toNativeAmount = _biggystring.round.call(void 0, toNativeAmountFloat, 0)
  log(`toNativeAmount: ${toNativeAmount}`)

  const memo = preMemo.replace(':0/', `:${toThorAmountWithSpread}/`)

  return {
    canBePartial,
    fromNativeAmount,
    fromExchangeAmount,
    maxFulfillmentSeconds,
    toNativeAmount,
    toExchangeAmount,
    memo,
    router,
    thorAddress
  }
}

const calcSwapTo = async ({
  log,
  fetch,
  thornodes,
  thornodesFetchOptions,
  fromWallet,
  fromCurrencyCode,
  toWallet,
  toCurrencyCode,
  toAddress,
  nativeAmount,
  minAmount,
  sourcePool,
  destPool,
  thorname,
  volatilitySpreadFinal,
  volatilitySpreadStreamingFinal,
  affiliateFeeBasis,
  streamingInterval,
  streamingQuantity,
  dontCheckLimits = false
}) => {
  const toNativeAmount = nativeAmount

  // Get exchange rate from source to destination asset
  const toExchangeAmount = await toWallet.nativeToDenomination(
    nativeAmount,
    toCurrencyCode
  )

  const requestedToThorAmount = _biggystring.mul.call(void 0, toExchangeAmount, exports.THOR_LIMIT_UNITS)

  log(`toExchangeAmount: ${toExchangeAmount}`)

  // Convert to a 'from' amount using pool rates
  const sourcePrice = sourcePool.assetPrice
  const destPrice = destPool.assetPrice

  const requestedFromExchangeAmount = _biggystring.mul.call(void 0, 
    toExchangeAmount,
    _biggystring.div.call(void 0, destPrice, sourcePrice, exports.DIVIDE_PRECISION)
  )

  const requestedFromThorAmount = _biggystring.round.call(void 0, 
    _biggystring.mul.call(void 0, requestedFromExchangeAmount, exports.THOR_LIMIT_UNITS),
    0
  )

  const noStreamParams = {
    amount: requestedFromThorAmount,
    from_asset: sourcePool.asset,
    to_asset: destPool.asset,
    destination: toAddress,
    affiliate: thorname,
    affiliate_bps: affiliateFeeBasis,
    streaming_interval: STREAMING_INTERVAL_NOSTREAM,
    streaming_quantity: STREAMING_QUANTITY_NOSTREAM
  }
  const streamParams = {
    ...noStreamParams,
    streaming_interval: streamingInterval,
    streaming_quantity: streamingQuantity
  }

  const bestQuote = await getBestQuote(
    [noStreamParams, streamParams],
    fetch,
    thornodes,
    thornodesFetchOptions
  )

  const {
    expected_amount_out_streaming: toThorAmount,
    inbound_address: thorAddress,
    memo: preMemo,
    router,
    streaming_swap_blocks: streamingSwapBlocks,
    total_swap_seconds: maxFulfillmentSeconds
  } = bestQuote

  const canBePartial = streamingSwapBlocks > 1

  // Get the percent drop from the 'to' amount the user wanted compared to the
  // 'to' amount returned by the API. Add that percent to the 'from' amount to
  // estimate how much more the user has to send.
  const feeRatio = _biggystring.div.call(void 0, requestedToThorAmount, toThorAmount, exports.DIVIDE_PRECISION)
  log(`feeRatio: ${feeRatio}`)

  const fromThorAmount = _biggystring.mul.call(void 0, requestedFromThorAmount, feeRatio)

  let fromThorAmountWithSpread
  if (canBePartial) {
    log(`volatilitySpreadStreamingFinal: ${volatilitySpreadStreamingFinal}`)
    fromThorAmountWithSpread = _biggystring.round.call(void 0, 
      _biggystring.mul.call(void 0, _biggystring.add.call(void 0, '1', volatilitySpreadStreamingFinal), fromThorAmount),
      0
    )
  } else {
    log(`volatilitySpreadFinal: ${volatilitySpreadFinal}`)
    fromThorAmountWithSpread = _biggystring.round.call(void 0, 
      _biggystring.mul.call(void 0, _biggystring.add.call(void 0, '1', volatilitySpreadFinal), fromThorAmount),
      0
    )
  }

  log(`fromThorAmountWithSpread = limit: ${fromThorAmountWithSpread}`)

  const fromExchangeAmount = _biggystring.div.call(void 0, 
    fromThorAmountWithSpread,
    exports.THOR_LIMIT_UNITS,
    exports.DIVIDE_PRECISION
  )
  log(`fromExchangeAmount: ${fromExchangeAmount}`)

  const fromNativeAmountFloat = await fromWallet.denominationToNative(
    fromExchangeAmount,
    fromCurrencyCode
  )
  const fromNativeAmount = _biggystring.round.call(void 0, fromNativeAmountFloat, 0)
  log(`fromNativeAmount: ${fromNativeAmount}`)

  const memo = preMemo.replace(':0/', `:${requestedToThorAmount}/`)

  return {
    canBePartial,
    fromNativeAmount,
    fromExchangeAmount,
    toNativeAmount,
    toExchangeAmount,
    maxFulfillmentSeconds,
    memo,
    router,
    thorAddress
  }
}











const getBestQuote = async (
  params,
  fetch,
  thornodes,
  thornodesFetchOptions
) => {
  const quotes = await Promise.all(
    params.map(
      async p => await getQuote(p, fetch, thornodes, thornodesFetchOptions)
    )
  )

  let bestQuote
  for (const quote of quotes) {
    if (quote == null) continue
    if (bestQuote == null) {
      bestQuote = quote
      continue
    }
    if (
      _biggystring.gt.call(void 0, 
        quote.expected_amount_out_streaming,
        bestQuote.expected_amount_out_streaming
      )
    ) {
      bestQuote = quote
    }
  }

  if (bestQuote == null) {
    throw new Error('Could not get quote')
  }
  return bestQuote
}

/**
 * getQuote must not throw!
 */
const getQuote = async (
  queryParams,
  fetch,
  thornodes,
  thornodesFetchOptions
) => {
  const params = _utils.makeQueryParams.call(void 0, queryParams)

  try {
    const response = await _utils.fetchWaterfall.call(void 0, 
      fetch,
      thornodes,
      `thorchain/quote/swap?${params}`,
      thornodesFetchOptions
    )
    let json
    try {
      json = await response.json()
    } catch (e) {
      const text = await response.text()
      console.error(text)
      return
    }
    console.log('cleaning')
    const quote = asQuoteSwap(json)
    return quote
  } catch (e) {
    console.error(`getQuote throw ${String(e)}`)
  }
}

 const getGasLimit = (
  chain,
  asset
) => {
  if (exports.EVM_CURRENCY_CODES[chain]) {
    if (chain === asset) {
      return exports.EVM_SEND_GAS
    } else {
      return exports.EVM_TOKEN_SEND_GAS
    }
  }
}; exports.getGasLimit = getGasLimit

 const getVolatilitySpread = ({
  fromPluginId,
  fromTokenId,
  fromCurrencyCode,
  toPluginId,
  toTokenId,
  toCurrencyCode,
  likeKindVolatilitySpread,
  volatilitySpread,
  perAssetSpread
}









) => {
  let volatilitySpreadFinal

  for (const spread of perAssetSpread) {
    const {
      sourcePluginId,
      sourceTokenId,
      sourceCurrencyCode,
      destPluginId,
      destTokenId,
      destCurrencyCode,
      volatilitySpread
    } = spread
    if (sourcePluginId != null && sourcePluginId !== fromPluginId) continue
    if (sourceTokenId != null && sourceTokenId !== fromTokenId) continue
    if (sourceCurrencyCode != null && sourceCurrencyCode !== fromCurrencyCode)
      continue
    if (destPluginId != null && destPluginId !== toPluginId) continue
    if (destTokenId != null && destTokenId !== toTokenId) continue
    if (destCurrencyCode != null && destCurrencyCode !== toCurrencyCode)
      continue
    volatilitySpreadFinal = volatilitySpread
    break
  }

  if (volatilitySpreadFinal == null) {
    const likeKind = _swaphelpers.isLikeKind.call(void 0, fromCurrencyCode, toCurrencyCode)

    volatilitySpreadFinal = likeKind
      ? likeKindVolatilitySpread
      : volatilitySpread
  }

  return volatilitySpreadFinal.toString()
}; exports.getVolatilitySpread = getVolatilitySpread
