"use strict";Object.defineProperty(exports, "__esModule", {value: true}); function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; } function _nullishCoalesce(lhs, rhsFn) { if (lhs != null) { return lhs; } else { return rhsFn(); } } function _optionalChain(ops) { let lastAccessLHS = undefined; let value = ops[0]; let i = 1; while (i < ops.length) { const op = ops[i]; const fn = ops[i + 1]; i += 2; if ((op === 'optionalAccess' || op === 'optionalCall') && value == null) { return undefined; } if (op === 'access' || op === 'optionalAccess') { lastAccessLHS = value; value = fn(value); } else if (op === 'call' || op === 'optionalCall') { value = fn((...args) => value.call(lastAccessLHS, ...args)); lastAccessLHS = undefined; } } return value; }








var _types = require('edge-core-js/types');
var _ethers = require('ethers');






var _swaphelpers = require('../../../../swap-helpers');
var _utils = require('../../../../util/utils');

var _VELODROME_V1_ROUTER_ABI = require('../../abi/VELODROME_V1_ROUTER_ABI'); var _VELODROME_V1_ROUTER_ABI2 = _interopRequireDefault(_VELODROME_V1_ROUTER_ABI);
var _WRAPPED_OPTIMISM_ETH_ABI = require('../../abi/WRAPPED_OPTIMISM_ETH_ABI'); var _WRAPPED_OPTIMISM_ETH_ABI2 = _interopRequireDefault(_WRAPPED_OPTIMISM_ETH_ABI);
var _defiUtils = require('../../defiUtils');
var _uniV2Utils = require('../uniV2Utils');

const swapInfo = {
  pluginId: 'velodrome',
  isDex: true,
  displayName: 'Velodrome',
  supportEmail: 'support@edge.app'
}

const EXPIRATION_MS = 1000 * 60
const SLIPPAGE = '0.05'
const OPTIMISM_RPC = 'https://rpc.ankr.com/optimism/'
const WETH_TOKEN_ADDRESS = '0x4200000000000000000000000000000000000006'
const VELODROME_ROUTER_ADDRESS = '0x9c12939390052919aF3155f41Bf4160Fd3666A6f'

 function makeVelodromePlugin(
  opts
) {
  const fetchSwapQuoteInner = async (
    request,
    uid
  ) => {
    const { fromWallet, toWallet, fromTokenId, toTokenId } = request

    if (
      // Velodrome does not support reverse quotes
      request.quoteFor === 'to' ||
      // Velodrome only supports Optimism
      fromWallet.currencyInfo.pluginId !== 'optimism' ||
      toWallet.currencyInfo.pluginId !== 'optimism'
    ) {
      throw new (0, _types.SwapCurrencyError)(swapInfo, request)
    }

    // Parse input/output token addresses. If either from or to swap sources
    // are for the native currency, convert the address to the wrapped equivalent.
    const inOutAddresses = _defiUtils.getInOutTokenAddresses.call(void 0, 
      fromWallet.currencyConfig,
      WETH_TOKEN_ADDRESS,
      fromTokenId,
      toTokenId
    )
    const { fromTokenAddress, toTokenAddress, isWrappingSwap } = inOutAddresses

    const provider = new _ethers.ethers.providers.JsonRpcProvider(OPTIMISM_RPC)
    // Calculate swap amounts
    const velodromeRouter = new _ethers.ethers.Contract(
      VELODROME_ROUTER_ADDRESS,
      _VELODROME_V1_ROUTER_ABI2.default,
      provider
    )

    // Identify best pool type
    const stable = isWrappingSwap
      ? true // No need to check for wrapping txs since it won't even use the dex
      : await velodromeRouter.getAmountOut(
          request.nativeAmount,
          fromTokenAddress,
          toTokenAddress
        ).stable

    const path = [[fromTokenAddress, toTokenAddress, stable]]

    const { amountToSwap, expectedAmountOut } = await _uniV2Utils.getSwapAmounts.call(void 0, 
      velodromeRouter,
      request,
      swapInfo,
      path,
      isWrappingSwap
    )

    // Generate swap transactions
    const toAddress = (await toWallet.getReceiveAddress()).publicAddress
    const expirationDate = new Date(Date.now() + EXPIRATION_MS)
    const deadline = Math.round(expirationDate.getTime() / 1000) // unix timestamp
    const customNetworkFee = _swaphelpers.customFeeCache.getFees(uid)
    const wrappedEthContract = new _ethers.ethers.Contract(
      WETH_TOKEN_ADDRESS,
      _WRAPPED_OPTIMISM_ETH_ABI2.default,
      provider
    )
    const swapTxs = await _uniV2Utils.getSwapTransactions.call(void 0, 
      provider,
      inOutAddresses,
      path,
      velodromeRouter,
      wrappedEthContract,
      amountToSwap,
      expectedAmountOut,
      toAddress,
      SLIPPAGE,
      deadline,
      _optionalChain([customNetworkFee, 'optionalAccess', _ => _.gasPrice])
    )
    const fromAddress = (await fromWallet.getReceiveAddress()).publicAddress
    // toEdgeUnsignedTxs
    const edgeSpendInfos = swapTxs.map(swapTx => {
      // Convert to our spendInfo
      const edgeSpendInfo = {
        currencyCode: request.fromCurrencyCode, // what is being sent out, only if token. Blank if not token
        spendTargets: [
          {
            memo: swapTx.data,
            nativeAmount: swapTx.value != null ? swapTx.value.toString() : '0', // biggy/number string integer
            publicAddress: swapTx.to
          }
        ],
        customNetworkFee: {
          gasPrice:
            swapTx.gasPrice != null
              ? _ethers.ethers.utils.formatUnits(swapTx.gasPrice, 'gwei').toString()
              : '0',
          gasLimit: _nullishCoalesce(_optionalChain([swapTx, 'access', _2 => _2.gasLimit, 'optionalAccess', _3 => _3.toString, 'call', _4 => _4()]), () => ( '0'))
        },
        networkFeeOption: 'custom',
        swapData: {
          isEstimate: false,
          payoutAddress: toAddress,
          payoutCurrencyCode: request.toCurrencyCode,
          payoutNativeAmount: expectedAmountOut.toString(),
          payoutWalletId: request.toWallet.id,
          plugin: { ...swapInfo },
          refundAddress: fromAddress
        }
      }

      return edgeSpendInfo
    })

    let spendInfo = edgeSpendInfos[0]
    let preTx
    if (edgeSpendInfos.length > 1) {
      spendInfo = edgeSpendInfos[1]
      edgeSpendInfos[0].metadata = { category: 'expense:Token Approval' }
      preTx = await request.fromWallet.makeSpend(edgeSpendInfos[0])
    }

    _swaphelpers.customFeeCache.setFees(uid, spendInfo.customNetworkFee)

    return {
      request,
      spendInfo,
      swapInfo,
      fromNativeAmount: amountToSwap,
      expirationDate,
      preTx
    }
  }

  const out = {
    swapInfo,
    async fetchSwapQuote(req) {
      const request = _utils.convertRequest.call(void 0, req)

      const uid = _swaphelpers.customFeeCache.createUid()

      const newRequest = await _swaphelpers.getMaxSwappable.call(void 0, 
        fetchSwapQuoteInner,
        request,
        uid
      )
      const swapOrder = await fetchSwapQuoteInner(newRequest, uid)
      return await _swaphelpers.makeSwapPluginQuote.call(void 0, swapOrder)
    }
  }

  return out
} exports.makeVelodromePlugin = makeVelodromePlugin;
