"use strict";Object.defineProperty(exports, "__esModule", {value: true});var _biggystring = require('biggystring');
var _cleaners = require('cleaners');









var _types = require('edge-core-js/types');
var _xrpl = require('xrpl');

var _swaphelpers = require('../../swap-helpers');






var _utils = require('../../util/utils');

var _xrpDexHelpers = require('./xrp/xrpDexHelpers');


const pluginId = 'xrpdex'
const swapInfo = {
  pluginId,
  isDex: true,
  displayName: 'XRP DEX',
  supportEmail: 'support@edge.app'
}

 const asInitOptions = _cleaners.asObject.call(void 0, {
  appId: _cleaners.asOptional.call(void 0, _cleaners.asString, 'edge')
}); exports.asInitOptions = asInitOptions

const EXPIRATION_MS = 1000 * 60
const DEX_MAX_FULLFILLMENT_TIME_S = 5 * 60 // 5 mins
const RIPPLE_SERVERS_DEFAULT = ['wss://s2.ripple.com']
const EXCHANGE_INFO_UPDATE_FREQ_MS = 60000
const VOLATILITY_SPREAD_DEFAULT = 0.0075
const DUMMY_XRP_ADDRESS = 'rfuESo7eHUnvebxgaFjfYxfwXhM2uBPAj3'

 const asExchangeInfo = _cleaners.asObject.call(void 0, {
  swap: _cleaners.asObject.call(void 0, {
    plugins: _cleaners.asObject.call(void 0, {
      xrpdex: _cleaners.asObject.call(void 0, {
        volatilitySpread: _cleaners.asNumber,
        rippleServers: _cleaners.asOptional.call(void 0, _cleaners.asArray.call(void 0, _cleaners.asString))
      })
    })
  })
}); exports.asExchangeInfo = asExchangeInfo



let exchangeInfo
let exchangeInfoLastUpdate = 0

 function makeXrpDexPlugin(opts) {
  const { io, log } = opts
  const { fetchCors = io.fetch } = io
  const { appId } = exports.asInitOptions.call(void 0, opts.initOptions)

  const fetchSwapQuoteInner = async (
    request
  ) => {
    const {
      fromCurrencyCode,
      fromTokenId,
      toCurrencyCode,
      toTokenId,
      nativeAmount,
      fromWallet,
      toWallet,
      quoteFor
    } = request

    // Only support ripple wallets
    if (
      fromWallet.currencyInfo.pluginId !== 'ripple' ||
      toWallet.currencyInfo.pluginId !== 'ripple'
    ) {
      throw new (0, _types.SwapCurrencyError)(swapInfo, request)
    }

    // Source and dest wallet must be the same
    if (fromWallet.id !== toWallet.id) {
      throw new Error('XRP DEX must use same wallet for source and destination')
    }

    // Do not support transfer between same assets
    if (request.fromTokenId === request.toTokenId) {
      throw new (0, _types.SwapCurrencyError)(swapInfo, request)
    }

    const rippleServers = RIPPLE_SERVERS_DEFAULT
    const volatilitySpread = VOLATILITY_SPREAD_DEFAULT

    const fromIssuer =
      fromTokenId != null ? fromTokenId.split('-')[1] : undefined
    const toIssuer = toTokenId != null ? toTokenId.split('-')[1] : undefined
    const fromCurrency =
      fromTokenId != null ? fromTokenId.split('-')[0] : fromCurrencyCode
    const toCurrency =
      toTokenId != null ? toTokenId.split('-')[0] : toCurrencyCode

    // Grab addresses:
    const toAddress = await _utils.getAddress.call(void 0, toWallet)

    const now = Date.now()
    if (
      now - exchangeInfoLastUpdate > EXCHANGE_INFO_UPDATE_FREQ_MS ||
      exchangeInfo == null
    ) {
      try {
        const exchangeInfoResponse = await _utils.promiseWithTimeout.call(void 0, 
          _utils.fetchInfo.call(void 0, fetchCors, `v1/exchangeInfo/${appId}`)
        )

        if (exchangeInfoResponse.ok === true) {
          exchangeInfo = exports.asExchangeInfo.call(void 0, await exchangeInfoResponse.json())
          exchangeInfoLastUpdate = now
        } else {
          // Error is ok. We just use defaults
          log('Error getting info server exchangeInfo. Using defaults...')
        }
      } catch (e) {
        log(
          'Error getting info server exchangeInfo. Using defaults...',
          e.message
        )
      }
    }

    // ----------------------------------------------------------------
    // Get a quote
    // ----------------------------------------------------------------

    const client = await getXrplConnectedClient(rippleServers)

    let quote
    let fromNativeAmount
    let toNativeAmount
    const taker = await _utils.getAddress.call(void 0, fromWallet)

    if (quoteFor === 'from') {
      fromNativeAmount = nativeAmount
      const exchangeAmount = await fromWallet.nativeToDenomination(
        nativeAmount,
        fromCurrencyCode
      )

      quote = await _xrpDexHelpers.getSellQuote.call(void 0, 
        {
          weSell: {
            currency: fromCurrency,
            issuer: fromIssuer
          },
          weSellAmountOfTokens: Number(exchangeAmount),
          counterCurrency: {
            currency: toCurrency,
            issuer: toIssuer
          },
          taker
        },
        { client, showLogs: false }
      )
      quote = quote * (1 - volatilitySpread)
      toNativeAmount = await toWallet.denominationToNative(
        String(quote),
        toCurrencyCode
      )
      toNativeAmount = _biggystring.round.call(void 0, toNativeAmount, 0)
    } else {
      toNativeAmount = nativeAmount
      const exchangeAmount = await toWallet.nativeToDenomination(
        nativeAmount,
        toCurrencyCode
      )

      quote = await _xrpDexHelpers.getBuyQuote.call(void 0, 
        {
          weWant: {
            currency: toCurrency,
            issuer: toIssuer
          },
          weWantAmountOfToken: Number(exchangeAmount),
          counterCurrency: {
            currency: fromCurrency,
            issuer: fromIssuer
          },
          taker
        },
        { client, showLogs: false }
      )
      quote = quote * (1 + volatilitySpread)
      fromNativeAmount = await fromWallet.denominationToNative(
        String(quote),
        fromCurrencyCode
      )
      fromNativeAmount = _biggystring.round.call(void 0, fromNativeAmount, 0)
    }

    const timestampNow = Date.now()
    const expiration = timestampNow / 1000 + DEX_MAX_FULLFILLMENT_TIME_S

    const swapData = {
      isEstimate: false,
      payoutAddress: toAddress,
      payoutCurrencyCode: toCurrencyCode,
      payoutNativeAmount: toNativeAmount,
      payoutWalletId: toWallet.id,
      plugin: { ...swapInfo }
    }

    const makeTxParams = {
      type: 'MakeTxDexSwap',
      metadata: {},
      swapData,
      fromTokenId,
      fromNativeAmount,
      toTokenId,
      toNativeAmount,
      expiration
    }

    await client.disconnect()

    return {
      canBePartial: true,
      maxFulfillmentSeconds: DEX_MAX_FULLFILLMENT_TIME_S,
      request,
      makeTxParams,
      swapInfo,
      fromNativeAmount,
      expirationDate: new Date(Date.now() + EXPIRATION_MS)
    }
  }

  const out = {
    swapInfo,

    async fetchSwapQuote(
      req,
      userSettings,
      opts
    ) {
      const request = _utils.convertRequest.call(void 0, req)
      const { fromCurrencyCode, fromTokenId, fromWallet, quoteFor } = request

      let swapOrder
      if (quoteFor === 'max') {
        request.quoteFor = 'from'
        request.nativeAmount = fromWallet.balances[fromCurrencyCode]
        swapOrder = await fetchSwapQuoteInner(request)
        if (fromTokenId == null) {
          // We can swap all mainnet coins minus the expected fee
          const quote = await _swaphelpers.makeSwapPluginQuote.call(void 0, swapOrder)
          const swapFee = quote.networkFee.nativeAmount

          // Get the balance of the wallet minus reserve
          const maxSpendable = await fromWallet.getMaxSpendable({
            currencyCode: fromCurrencyCode,
            spendTargets: [
              {
                publicAddress: DUMMY_XRP_ADDRESS
              }
            ]
          })
          request.nativeAmount = _biggystring.sub.call(void 0, maxSpendable, swapFee)
          return await this.fetchSwapQuote(request, userSettings, opts)
        }
      } else {
        swapOrder = await fetchSwapQuoteInner(request)
      }
      return await _swaphelpers.makeSwapPluginQuote.call(void 0, swapOrder)
    }
  }
  return out
} exports.makeXrpDexPlugin = makeXrpDexPlugin;

const getXrplConnectedClient = async (servers) => {
  let client
  let error
  if (servers.length === 0) {
    throw new Error('No ripple servers')
  }
  const shuffled = _utils.shuffleArray.call(void 0, servers)
  for (const server of shuffled) {
    client = new (0, _xrpl.Client)(server)
    try {
      await client.connect()
      return client
    } catch (e) {
      error = e
      // Harmless if one server fails
    }
  }
  throw error
}
