"use strict";Object.defineProperty(exports, "__esModule", {value: true});var _biggystring = require('biggystring');







var _cleaners = require('cleaners');









var _types = require('edge-core-js/types');








var _swaphelpers = require('../swap-helpers');
var _utils = require('../util/utils');


const pluginId = 'exolix'

const swapInfo = {
  pluginId,
  isDex: false,
  displayName: 'Exolix',
  supportEmail: 'support@exolix.com'
}

const asInitOptions = _cleaners.asObject.call(void 0, {
  apiKey: _cleaners.asString
})

const INVALID_CURRENCY_CODES = {
  from: {
    binancesmartchain: 'allCodes',
    ethereum: ['MATIC'],
    optimism: 'allCodes',
    polygon: 'allCodes'
  },
  to: {
    binancesmartchain: 'allCodes',
    ethereum: ['MATIC'],
    optimism: 'allCodes',
    polygon: 'allCodes',
    zcash: ['ZEC']
  }
}

// See https://exolix.com/currencies for list of supported currencies
const MAINNET_CODE_TRANSCRIPTION = {}

const orderUri = 'https://exolix.com/transaction/'
const uri = 'https://exolix.com/api/v2/'

const expirationMs = 1000 * 60

const asRateResponse = _cleaners.asObject.call(void 0, {
  minAmount: _cleaners.asNumber,
  withdrawMin: _cleaners.asOptional.call(void 0, _cleaners.asNumber, 0),
  fromAmount: _cleaners.asNumber,
  toAmount: _cleaners.asNumber,
  message: _cleaners.asEither.call(void 0, _cleaners.asString, _cleaners.asNull)
})

const asQuoteInfo = _cleaners.asObject.call(void 0, {
  id: _cleaners.asString,
  amount: _cleaners.asNumber,
  amountTo: _cleaners.asNumber,
  depositAddress: _cleaners.asString,
  depositExtraId: _cleaners.asOptional.call(void 0, _cleaners.asString)
})

 function makeExolixPlugin(opts) {
  const { io, log } = opts
  const { fetchCors = io.fetch } = io
  const { apiKey } = asInitOptions(opts.initOptions)

  async function call(
    method,
    route,
    params
  ) {
    const headers = {
      Accept: 'application/json',
      'Content-Type': 'application/json',
      Authorization: `${apiKey}`
    }

    let response

    if (method === 'POST') {
      const body = JSON.stringify(params)
      response = await fetchCors(uri + route, {
        method,
        headers,
        body
      })
    } else {
      const url = `${uri}${route}?${new URLSearchParams(params).toString()}`
      response = await fetchCors(url, {
        method,
        headers
      })
    }

    if (
      !response.ok &&
      !(await response.text()).includes(
        'Amount to exchange is below the possible min amount to exchange'
      ) // HACK: Exolix inconsistently returns a !ok response for a 'from' quote
      // under minimum amount, while the status is OK for a 'to' quote under
      // minimum amount.
      // Handle this inconsistency and ensure parse the proper under min error
      // and we don't exit early with the wrong 'unsupported' error message.
    ) {
      log.warn(`Error retrieving Exolix quote: ${await response.text()}`)
      if (response.status === 422) {
        throw new (0, _types.SwapCurrencyError)(swapInfo, params.coinFrom, params.coinTo)
      }
      throw new Error(`Exolix returned error code ${response.status}`)
    }

    return await response.json()
  }

  const getFixedQuote = async (
    request,
    _userSettings
  ) => {
    const [fromAddress, toAddress] = await Promise.all([
      _utils.getAddress.call(void 0, request.fromWallet),
      _utils.getAddress.call(void 0, request.toWallet)
    ])

    const exchangeQuoteAmount =
      request.quoteFor === 'from'
        ? await request.fromWallet.nativeToDenomination(
            request.nativeAmount,
            request.fromCurrencyCode
          )
        : await request.toWallet.nativeToDenomination(
            request.nativeAmount,
            request.toCurrencyCode
          )

    const quoteAmount = parseFloat(exchangeQuoteAmount)

    const {
      fromCurrencyCode,
      toCurrencyCode,
      fromMainnetCode,
      toMainnetCode
    } = _swaphelpers.getCodesWithTranscription.call(void 0, request, MAINNET_CODE_TRANSCRIPTION)

    const quoteParams = {
      coinFrom: fromCurrencyCode,
      coinFromNetwork: fromMainnetCode,
      coinTo: toCurrencyCode,
      coinToNetwork: toMainnetCode,
      amount: quoteAmount,
      rateType: 'fixed'
    }

    // Set the withdrawal amount if we are quoting for the toCurrencyCode
    if (request.quoteFor === 'to') {
      quoteParams.withdrawalAmount = quoteAmount
    }

    // Get Rate
    const rateResponse = asRateResponse(await call('GET', 'rate', quoteParams))

    // Check rate minimum:
    if (request.quoteFor === 'from') {
      const nativeMin = await request.fromWallet.denominationToNative(
        rateResponse.minAmount.toString(),
        request.fromCurrencyCode
      )

      if (_biggystring.lt.call(void 0, request.nativeAmount, nativeMin)) {
        throw new (0, _types.SwapBelowLimitError)(swapInfo, nativeMin, 'from')
      }
    } else {
      const nativeMin = await request.toWallet.denominationToNative(
        rateResponse.withdrawMin.toString(),
        request.toCurrencyCode
      )

      if (_biggystring.lt.call(void 0, request.nativeAmount, nativeMin)) {
        throw new (0, _types.SwapBelowLimitError)(swapInfo, nativeMin, 'to')
      }
    }

    // Make the transaction:
    const exchangeParams = {
      coinFrom: quoteParams.coinFrom,
      networkFrom: quoteParams.coinFromNetwork,
      coinTo: quoteParams.coinTo,
      networkTo: quoteParams.coinToNetwork,
      amount: quoteAmount,
      withdrawalAddress: toAddress,
      withdrawalExtraId: '',
      refundAddress: fromAddress,
      refundExtraId: '',
      rateType: 'fixed'
    }

    // Set the withdrawal amount if we are quoting for the toCurrencyCode
    if (request.quoteFor === 'to') {
      exchangeParams.withdrawalAmount = quoteAmount
    }

    const callJson = await call('POST', 'transactions', exchangeParams)
    const quoteInfo = asQuoteInfo(callJson)

    const fromNativeAmount = await request.fromWallet.denominationToNative(
      quoteInfo.amount.toString(),
      request.fromCurrencyCode
    )

    const toNativeAmount = await request.toWallet.denominationToNative(
      quoteInfo.amountTo.toString(),
      request.toCurrencyCode
    )

    const spendInfo = {
      currencyCode: request.fromCurrencyCode,
      spendTargets: [
        {
          nativeAmount: fromNativeAmount,
          publicAddress: quoteInfo.depositAddress,
          uniqueIdentifier: quoteInfo.depositExtraId
        }
      ],
      networkFeeOption:
        request.fromCurrencyCode.toUpperCase() === 'BTC' ? 'high' : 'standard',
      swapData: {
        orderId: quoteInfo.id,
        orderUri: orderUri + quoteInfo.id,
        isEstimate: false,
        payoutAddress: toAddress,
        payoutCurrencyCode: request.toCurrencyCode,
        payoutNativeAmount: toNativeAmount,
        payoutWalletId: request.toWallet.id,
        plugin: {
          ...swapInfo
        },
        refundAddress: fromAddress
      }
    }

    return {
      request,
      spendInfo,
      swapInfo,
      fromNativeAmount,
      expirationDate: new Date(Date.now() + expirationMs)
    }
  }

  const out = {
    swapInfo,
    async fetchSwapQuote(
      req,
      userSettings
    ) {
      const request = _utils.convertRequest.call(void 0, req)

      _swaphelpers.checkInvalidCodes.call(void 0, INVALID_CURRENCY_CODES, request, swapInfo)

      const newRequest = await _swaphelpers.getMaxSwappable.call(void 0, 
        getFixedQuote,
        request,
        userSettings
      )
      const fixedOrder = await getFixedQuote(newRequest, userSettings)
      const fixedResult = await _swaphelpers.makeSwapPluginQuote.call(void 0, fixedOrder)

      return fixedResult
    }
  }

  return out
} exports.makeExolixPlugin = makeExolixPlugin;
