"use strict";Object.defineProperty(exports, "__esModule", {value: true}); function _nullishCoalesce(lhs, rhsFn) { if (lhs != null) { return lhs; } else { return rhsFn(); } } function _optionalChain(ops) { let lastAccessLHS = undefined; let value = ops[0]; let i = 1; while (i < ops.length) { const op = ops[i]; const fn = ops[i + 1]; i += 2; if ((op === 'optionalAccess' || op === 'optionalCall') && value == null) { return undefined; } if (op === 'access' || op === 'optionalAccess') { lastAccessLHS = value; value = fn(value); } else if (op === 'call' || op === 'optionalCall') { value = fn((...args) => value.call(lastAccessLHS, ...args)); lastAccessLHS = undefined; } } return value; }var _biggystring = require('biggystring');








var _cleaners = require('cleaners');










var _types = require('edge-core-js/types');








var _swaphelpers = require('../swap-helpers');
var _utils = require('../util/utils');
var _types3 = require('./types');

const pluginId = 'godex'

const swapInfo = {
  pluginId,
  isDex: false,
  displayName: 'Godex',
  supportEmail: 'support@godex.io'
}

const asInitOptions = _cleaners.asObject.call(void 0, {
  apiKey: _cleaners.asOptional.call(void 0, _cleaners.asString)
})

const orderUri = 'https://godex.io/exchange/waiting/'
const uri = 'https://api.godex.io/api/v1/'

const expirationMs = 1000 * 60

const asApiInfo = _cleaners.asObject.call(void 0, {
  min_amount: _types3.asNumberString,
  networks_from: _cleaners.asMaybe.call(void 0, 
    _cleaners.asArray.call(void 0, 
      _cleaners.asObject.call(void 0, {
        network: _cleaners.asString
      })
    )
  ),
  networks_to: _cleaners.asMaybe.call(void 0, 
    _cleaners.asArray.call(void 0, 
      _cleaners.asObject.call(void 0, {
        network: _cleaners.asString
      })
    )
  )
})

const asQuoteInfo = _cleaners.asObject.call(void 0, {
  transaction_id: _cleaners.asString,
  deposit: _cleaners.asString,
  deposit_extra_id: _cleaners.asEither.call(void 0, _cleaners.asString, _cleaners.asNull),
  deposit_amount: _cleaners.asString,
  withdrawal: _cleaners.asString,
  withdrawal_extra_id: _cleaners.asEither.call(void 0, _cleaners.asString, _cleaners.asNull),
  withdrawal_amount: _cleaners.asString,
  return: _cleaners.asString,
  return_extra_id: _cleaners.asEither.call(void 0, _cleaners.asString, _cleaners.asNull)
})

const INVALID_CURRENCY_CODES = {
  from: {
    avalanche: 'allTokens',
    celo: 'allTokens',
    digibyte: 'allCodes',
    ethereum: ['MATIC'],
    fantom: 'allTokens',
    optimism: ['VELO'],
    polygon: 'allCodes'
  },
  to: {
    avalanche: 'allTokens',
    celo: 'allTokens',
    ethereum: ['MATIC'],
    fantom: 'allTokens',
    polygon: 'allCodes',
    zcash: ['ZEC']
  }
}

// Network names that don't match parent network currency code
// See https://godex.io/exchange-rate for list of supported currencies
const MAINNET_CODE_TRANSCRIPTION = {
  rsk: 'RSK',
  binancesmartchain: 'BSC',
  avalanche: 'AVAXC'
}

 function makeGodexPlugin(opts) {
  const { io, log } = opts
  const { fetchCors = io.fetch } = io
  const initOptions = asInitOptions(opts.initOptions)

  async function call(
    url,
    request,
    data
  ) {
    const body = JSON.stringify(data.params)

    const headers = {
      'Content-Type': 'application/json',
      Accept: 'application/json'
    }
    const response = await fetchCors(url, { method: 'POST', body, headers })
    if (!response.ok) {
      if (response.status === 422) {
        throw new (0, _types.SwapCurrencyError)(
          swapInfo,
          request.fromCurrencyCode,
          request.toCurrencyCode
        )
      }
      throw new Error(`godex returned error code ${response.status}`)
    }
    return await response.json()
  }

  const fetchSwapQuoteInner = async (
    request,
    opts
  ) => {
    const reverseQuote = request.quoteFor === 'to'

    // Grab addresses:
    const [fromAddress, toAddress] = await Promise.all([
      _utils.getAddress.call(void 0, request.fromWallet),
      _utils.getAddress.call(void 0, request.toWallet)
    ])

    const { fromMainnetCode, toMainnetCode } = _swaphelpers.getCodesWithTranscription.call(void 0, 
      request,
      MAINNET_CODE_TRANSCRIPTION
    )

    // Convert the native amount to a denomination:
    const quoteAmount =
      request.quoteFor === 'from'
        ? await request.fromWallet.nativeToDenomination(
            request.nativeAmount,
            request.fromCurrencyCode
          )
        : await request.toWallet.nativeToDenomination(
            request.nativeAmount,
            request.toCurrencyCode
          )

    // Swap the currencies if we need a reverse quote:
    const quoteParams = {
      from: request.fromCurrencyCode,
      to: request.toCurrencyCode,
      amount: quoteAmount
    }
    log('quoteParams:', quoteParams)

    // Check if we are below the minimum limit:
    let fromAmount, toAmount, endpoint
    if (request.quoteFor === 'from') {
      fromAmount = quoteAmount
      endpoint = 'info'
    } else {
      toAmount = quoteAmount
      endpoint = 'info-revert'
    }
    const response = await call(uri + endpoint, request, {
      params: quoteParams
    })
    const reply = asApiInfo(response)

    // The info/info-revert endpoints don't accept the coin_from/coin_to params so the
    // min_amount returned could be for a different network than the user is requesting.
    // Check the minimum:
    const nativeMin = reverseQuote
      ? await request.toWallet.denominationToNative(
          reply.min_amount,
          request.toCurrencyCode
        )
      : await request.fromWallet.denominationToNative(
          reply.min_amount,
          request.fromCurrencyCode
        )

    if (_biggystring.lt.call(void 0, request.nativeAmount, nativeMin)) {
      throw new (0, _types.SwapBelowLimitError)(
        swapInfo,
        nativeMin,
        reverseQuote ? 'to' : 'from'
      )
    }

    // Check the networks. Networks aren't present for disabled assets.
    if (
      _optionalChain([reply, 'access', _ => _.networks_from, 'optionalAccess', _2 => _2.find, 'call', _3 => _3(
        network => network.network === fromMainnetCode
      )]) == null ||
      _optionalChain([reply, 'access', _4 => _4.networks_to, 'optionalAccess', _5 => _5.find, 'call', _6 => _6(network => network.network === toMainnetCode)]) ==
        null
    ) {
      throw new (0, _types.SwapCurrencyError)(
        swapInfo,
        request.fromCurrencyCode,
        request.toCurrencyCode
      )
    }

    const { promoCode } = opts

    endpoint = reverseQuote ? 'transaction-revert' : 'transaction'
    const sendReply = await call(
      uri + endpoint + (promoCode != null ? `?promo=${promoCode}` : ''),
      request,
      {
        params: {
          deposit_amount: reverseQuote ? undefined : fromAmount,
          withdrawal_amount: reverseQuote ? toAmount : undefined,
          coin_from: request.fromCurrencyCode,
          coin_to: request.toCurrencyCode,
          withdrawal: toAddress,
          return: fromAddress,
          return_extra_id: null,
          withdrawal_extra_id: null,
          affiliate_id: initOptions.apiKey,
          type: 'edge',
          isEstimate: false,
          coin_from_network: fromMainnetCode,
          coin_to_network: toMainnetCode
        }
      }
    )
    log('sendReply' + JSON.stringify(sendReply, null, 2))
    const quoteInfo = asQuoteInfo(sendReply)
    const fromNativeAmount = await request.fromWallet.denominationToNative(
      quoteInfo.deposit_amount,
      request.fromCurrencyCode
    )
    const toNativeAmount = await request.toWallet.denominationToNative(
      quoteInfo.withdrawal_amount,
      request.toCurrencyCode
    )

    log('fromNativeAmount: ' + fromNativeAmount)
    log('toNativeAmount: ' + toNativeAmount)

    // Make the transaction:
    const spendInfo = {
      currencyCode: request.fromCurrencyCode,
      spendTargets: [
        {
          nativeAmount: fromNativeAmount,
          publicAddress: quoteInfo.deposit,
          uniqueIdentifier: _nullishCoalesce(quoteInfo.deposit_extra_id, () => ( undefined))
        }
      ],
      networkFeeOption:
        request.fromCurrencyCode.toUpperCase() === 'BTC' ? 'high' : 'standard',
      swapData: {
        orderId: quoteInfo.transaction_id,
        orderUri: orderUri + quoteInfo.transaction_id,
        isEstimate: false,
        payoutAddress: toAddress,
        payoutCurrencyCode: request.toCurrencyCode,
        payoutNativeAmount: toNativeAmount,
        payoutWalletId: request.toWallet.id,
        plugin: { ...swapInfo },
        refundAddress: fromAddress
      }
    }
    log('spendInfo', spendInfo)

    return {
      request,
      spendInfo,
      swapInfo,
      fromNativeAmount,
      expirationDate: new Date(Date.now() + expirationMs)
    }
  }

  const out = {
    swapInfo,

    async fetchSwapQuote(
      req,
      userSettings,
      opts
    ) {
      const request = _utils.convertRequest.call(void 0, req)
      _swaphelpers.checkInvalidCodes.call(void 0, INVALID_CURRENCY_CODES, request, swapInfo)

      const newRequest = await _swaphelpers.getMaxSwappable.call(void 0, 
        fetchSwapQuoteInner,
        request,
        opts
      )
      const swapOrder = await fetchSwapQuoteInner(newRequest, opts)
      return await _swaphelpers.makeSwapPluginQuote.call(void 0, swapOrder)
    }
  }

  return out
} exports.makeGodexPlugin = makeGodexPlugin;
