"use strict";Object.defineProperty(exports, "__esModule", {value: true}); function _optionalChain(ops) { let lastAccessLHS = undefined; let value = ops[0]; let i = 1; while (i < ops.length) { const op = ops[i]; const fn = ops[i + 1]; i += 2; if ((op === 'optionalAccess' || op === 'optionalCall') && value == null) { return undefined; } if (op === 'access' || op === 'optionalAccess') { lastAccessLHS = value; value = fn(value); } else if (op === 'call' || op === 'optionalCall') { value = fn((...args) => value.call(lastAccessLHS, ...args)); lastAccessLHS = undefined; } } return value; }var _biggystring = require('biggystring');
var _cleaners = require('cleaners');










var _types = require('edge-core-js/types');








var _swaphelpers = require('../swap-helpers');
var _utils = require('../util/utils');
var _changenow = require('./changenow');
var _types3 = require('./types');

const pluginId = 'letsexchange'

const swapInfo = {
  pluginId,
  isDex: false,
  displayName: 'LetsExchange',
  supportEmail: 'support@letsexchange.io'
}

const asInitOptions = _cleaners.asObject.call(void 0, {
  apiKey: _cleaners.asString,
  affiliateId: _cleaners.asOptional.call(void 0, _cleaners.asString)
})

const orderUri = 'https://letsexchange.io/?exchangeId='
const uri = 'https://api.letsexchange.io/api/v1/'

const expirationMs = 1000 * 60

const asQuoteInfo = _cleaners.asObject.call(void 0, {
  transaction_id: _cleaners.asString,
  deposit_amount: _cleaners.asString,
  deposit: _cleaners.asString,
  deposit_extra_id: _changenow.asOptionalBlank.call(void 0, _cleaners.asString),
  withdrawal_amount: _cleaners.asString,
  withdrawal_extra_id: _changenow.asOptionalBlank.call(void 0, _cleaners.asString)
})

const asInfoReply = _cleaners.asObject.call(void 0, {
  min_amount: _types3.asNumberString,
  max_amount: _types3.asNumberString,
  amount: _types3.asNumberString
})
const INVALID_CURRENCY_CODES = {
  from: {
    ethereum: ['MATH'],
    optimism: ['VELO']
  },
  to: {
    ethereum: ['MATH'],
    zcash: ['ZEC']
  }
}

// See https://letsexchange.io/exchange-pairs for list of supported currencies
const MAINNET_CODE_TRANSCRIPTION = {
  avalanche: 'AVAXC',
  binance: 'BEP2',
  binancesmartchain: 'BEP20',
  ethereum: 'ERC20',
  optimism: 'OPTIMISM',
  rsk: 'RSK',
  tron: 'TRC20'
}

const SPECIAL_MAINNET_CASES = {
  avalanche: {
    AVAX: 'AVAXC'
  },
  binancesmartchain: {
    BNB: 'BEP20'
  },
  optimism: {
    ETH: 'OPTIMISM'
  },
  zksync: {
    ETH: 'ZKSYNC'
  }
}

 function makeLetsExchangePlugin(
  opts
) {
  const { io, log } = opts
  const { fetchCors = io.fetch } = io
  const initOptions = asInitOptions(opts.initOptions)

  async function call(
    url,
    request,
    data
  ) {
    const body = JSON.stringify(data.params)

    const headers = {
      'Content-Type': 'application/json',
      Authorization: `Bearer ${initOptions.apiKey}`,
      Accept: 'application/json'
    }
    const response = await fetchCors(url, { method: 'POST', body, headers })
    if (!response.ok) {
      if (response.status === 422) {
        throw new (0, _types.SwapCurrencyError)(
          swapInfo,
          request.fromCurrencyCode,
          request.toCurrencyCode
        )
      }
      throw new Error(`letsexchange returned error code ${response.status}`)
    }
    return await response.json()
  }

  const fetchSwapQuoteInner = async (
    request,
    opts
  ) => {
    const reverseQuote = request.quoteFor === 'to'

    // Grab addresses:
    const [fromAddress, toAddress] = await Promise.all([
      _utils.getAddress.call(void 0, request.fromWallet),
      _utils.getAddress.call(void 0, request.toWallet)
    ])

    // Convert the native amount to a denomination:
    const quoteAmount =
      request.quoteFor === 'from'
        ? await request.fromWallet.nativeToDenomination(
            request.nativeAmount,
            request.fromCurrencyCode
          )
        : await request.toWallet.nativeToDenomination(
            request.nativeAmount,
            request.toCurrencyCode
          )

    const { fromMainnetCode, toMainnetCode } = _swaphelpers.getCodesWithTranscription.call(void 0, 
      request,
      MAINNET_CODE_TRANSCRIPTION
    )

    const { pluginId: fromPluginId } = request.fromWallet.currencyInfo
    const networkFrom =
      _optionalChain([SPECIAL_MAINNET_CASES, 'access', _ => _[fromPluginId], 'optionalAccess', _2 => _2[request.toCurrencyCode]]) != null
        ? SPECIAL_MAINNET_CASES[fromPluginId][request.toCurrencyCode]
        : request.fromCurrencyCode ===
          request.fromWallet.currencyInfo.currencyCode
        ? request.fromCurrencyCode
        : fromMainnetCode

    const { pluginId: toPluginId } = request.toWallet.currencyInfo
    const networkTo =
      _optionalChain([SPECIAL_MAINNET_CASES, 'access', _3 => _3[toPluginId], 'optionalAccess', _4 => _4[request.toCurrencyCode]]) != null
        ? SPECIAL_MAINNET_CASES[toPluginId][request.toCurrencyCode]
        : request.toCurrencyCode === request.toWallet.currencyInfo.currencyCode
        ? request.toCurrencyCode
        : toMainnetCode

    // Swap the currencies if we need a reverse quote:
    const quoteParams = {
      from: request.fromCurrencyCode,
      to: request.toCurrencyCode,
      network_from: networkFrom,
      network_to: networkTo,
      amount: quoteAmount
    }

    log('quoteParams:', quoteParams)

    // Calculate the amounts:
    let fromAmount, toAmount, endpoint
    if (request.quoteFor === 'from') {
      fromAmount = quoteAmount
      endpoint = 'info'
    } else {
      toAmount = quoteAmount
      endpoint = 'info-revert'
    }
    const response = await call(uri + endpoint, request, {
      params: quoteParams
    })
    const reply = asInfoReply(response)

    // Check the min/max:
    const nativeMin = reverseQuote
      ? await request.toWallet.denominationToNative(
          reply.min_amount,
          request.toCurrencyCode
        )
      : await request.fromWallet.denominationToNative(
          reply.min_amount,
          request.fromCurrencyCode
        )

    if (_biggystring.lt.call(void 0, request.nativeAmount, nativeMin)) {
      throw new (0, _types.SwapBelowLimitError)(
        swapInfo,
        nativeMin,
        reverseQuote ? 'to' : 'from'
      )
    }

    const nativeMax = reverseQuote
      ? await request.toWallet.denominationToNative(
          reply.max_amount,
          request.toCurrencyCode
        )
      : await request.fromWallet.denominationToNative(
          reply.max_amount,
          request.fromCurrencyCode
        )

    if (_biggystring.gt.call(void 0, nativeMax, '0')) {
      if (_biggystring.gt.call(void 0, request.nativeAmount, nativeMax)) {
        throw new (0, _types.SwapAboveLimitError)(
          swapInfo,
          nativeMin,
          reverseQuote ? 'to' : 'from'
        )
      }
    }

    const { promoCode } = opts
    endpoint = reverseQuote ? 'transaction-revert' : 'transaction'
    const sendReply = await call(uri + endpoint, request, {
      params: {
        deposit_amount: reverseQuote ? undefined : fromAmount,
        withdrawal_amount: reverseQuote ? toAmount : undefined,
        coin_from: request.fromCurrencyCode,
        coin_to: request.toCurrencyCode,
        network_from: networkFrom,
        network_to: networkTo,
        withdrawal: toAddress,
        return: fromAddress,
        return_extra_id: null,
        withdrawal_extra_id: null,
        affiliate_id: initOptions.affiliateId,
        promocode: promoCode != null ? promoCode : '',
        type: 'edge',
        float: false,
        isEstimate: false
      }
    })

    log('sendReply', sendReply)
    const quoteInfo = asQuoteInfo(sendReply)

    const fromNativeAmount = await request.fromWallet.denominationToNative(
      quoteInfo.deposit_amount,
      request.fromCurrencyCode
    )
    const toNativeAmount = await request.toWallet.denominationToNative(
      quoteInfo.withdrawal_amount,
      request.toCurrencyCode
    )

    // Make the transaction:
    const spendInfo = {
      currencyCode: request.fromCurrencyCode,
      spendTargets: [
        {
          nativeAmount: fromNativeAmount,
          publicAddress: quoteInfo.deposit,
          uniqueIdentifier: quoteInfo.deposit_extra_id
        }
      ],
      networkFeeOption:
        request.fromCurrencyCode.toUpperCase() === 'BTC' ? 'high' : 'standard',
      swapData: {
        orderId: quoteInfo.transaction_id,
        orderUri: orderUri + quoteInfo.transaction_id,
        isEstimate: false,
        payoutAddress: toAddress,
        payoutCurrencyCode: request.toCurrencyCode,
        payoutNativeAmount: toNativeAmount,
        payoutWalletId: request.toWallet.id,
        plugin: { ...swapInfo },
        refundAddress: fromAddress
      }
    }

    log('spendInfo', spendInfo)

    return {
      request,
      spendInfo,
      swapInfo,
      fromNativeAmount,
      expirationDate: new Date(Date.now() + expirationMs)
    }
  }

  const out = {
    swapInfo,

    async fetchSwapQuote(
      req,
      userSettings,
      opts
    ) {
      const request = _utils.convertRequest.call(void 0, req)
      _swaphelpers.checkInvalidCodes.call(void 0, INVALID_CURRENCY_CODES, request, swapInfo)

      const newRequest = await _swaphelpers.getMaxSwappable.call(void 0, 
        fetchSwapQuoteInner,
        request,
        opts
      )
      const swapOrder = await fetchSwapQuoteInner(newRequest, opts)
      return await _swaphelpers.makeSwapPluginQuote.call(void 0, swapOrder)
    }
  }

  return out
} exports.makeLetsExchangePlugin = makeLetsExchangePlugin;
