"use strict";Object.defineProperty(exports, "__esModule", {value: true}); function _optionalChain(ops) { let lastAccessLHS = undefined; let value = ops[0]; let i = 1; while (i < ops.length) { const op = ops[i]; const fn = ops[i + 1]; i += 2; if ((op === 'optionalAccess' || op === 'optionalCall') && value == null) { return undefined; } if (op === 'access' || op === 'optionalAccess') { lastAccessLHS = value; value = fn(value); } else if (op === 'call' || op === 'optionalCall') { value = fn((...args) => value.call(lastAccessLHS, ...args)); lastAccessLHS = undefined; } } return value; }var _biggystring = require('biggystring');







var _cleaners = require('cleaners');









var _types = require('edge-core-js/types');










var _swaphelpers = require('../swap-helpers');
var _biggystringplus = require('../util/biggystringplus');
var _utils = require('../util/utils');


const pluginId = 'swapuz'

const swapInfo = {
  pluginId,
  isDex: false,
  displayName: 'Swapuz',
  supportEmail: 'support@swapuz.com'
}

const asInitOptions = _cleaners.asObject.call(void 0, {
  apiKey: _cleaners.asString
})

const orderUri = 'https://swapuz.com/order/'
const uri = 'https://api.swapuz.com/api/home/v1/'

const INVALID_CURRENCY_CODES = {
  from: {
    ethereum: ['MATH'],
    optimism: ['VELO']
  },
  to: {
    ethereum: ['MATH'],
    zcash: ['ZEC'],
    zksync: 'allCodes'
  }
}

// Network names that don't match parent network currency code
// See https://swapuz.com/ for list of supported currencies
const MAINNET_CODE_TRANSCRIPTION = {
  binancesmartchain: 'BSC',
  optimism: 'OPTIMISM'
}

 function makeSwapuzPlugin(opts) {
  const { io } = opts
  const { fetchCors = io.fetch } = io
  const { apiKey } = asInitOptions(opts.initOptions)

  const headers = {
    'Content-Type': 'application/json',
    'api-key': apiKey
  }

  const fetchSwapQuoteInner = async (
    request
  ) => {
    const { fromWallet, toWallet } = request

    _swaphelpers.checkInvalidCodes.call(void 0, INVALID_CURRENCY_CODES, request, swapInfo)

    // Grab addresses:
    const [fromAddress, toAddress] = await Promise.all([
      _utils.getAddress.call(void 0, fromWallet),
      _utils.getAddress.call(void 0, toWallet)
    ])

    const {
      fromCurrencyCode,
      toCurrencyCode,
      fromMainnetCode,
      toMainnetCode
    } = _swaphelpers.getCodesWithTranscription.call(void 0, request, MAINNET_CODE_TRANSCRIPTION)

    const getQuote = async (mode) => {
      const { nativeAmount } = request

      const largeDenomAmount = await fromWallet.nativeToDenomination(
        nativeAmount,
        fromCurrencyCode
      )

      const getRateResponse = await fetchCors(
        uri +
          `rate/?mode=${mode}&amount=${largeDenomAmount}&from=${fromCurrencyCode}&to=${toCurrencyCode}&fromNetwork=${fromMainnetCode}&toNetwork=${toMainnetCode}`,
        { headers }
      )
      if (!getRateResponse.ok) {
        throw new Error(
          `Swapuz call returned error code ${getRateResponse.status}`
        )
      }

      const getRateJson = asApiResponse(asGetRate)(await getRateResponse.json())

      if (getRateJson.result == null)
        throw new (0, _types.SwapCurrencyError)(swapInfo, fromCurrencyCode, toCurrencyCode)

      const { minAmount } = getRateJson.result

      if (_biggystring.gt.call(void 0, minAmount.toString(), largeDenomAmount)) {
        const nativeMinAmount = await fromWallet.denominationToNative(
          minAmount.toString(),
          fromCurrencyCode
        )
        throw new (0, _types.SwapBelowLimitError)(swapInfo, nativeMinAmount)
      }

      // Create order
      const orderBody = {
        from: fromCurrencyCode,
        fromNetwork: fromMainnetCode,
        to: toCurrencyCode,
        toNetwork: toMainnetCode,
        address: toAddress,
        amount: parseFloat(largeDenomAmount),
        mode,
        addressUserFrom: fromAddress,
        addressRefund: fromAddress
      }

      const createOrderResponse = await fetchCors(uri + 'order', {
        method: 'POST',
        body: JSON.stringify(orderBody),
        headers
      })
      if (!createOrderResponse.ok) {
        const text = await createOrderResponse.text()
        throw new Error(
          `Swapuz call returned error code ${createOrderResponse.status}\n${text}`
        )
      }

      const createOrderJson = asApiResponse(asCreateOrder)(
        await createOrderResponse.json()
      )

      if (createOrderJson.result == null) {
        throw new (0, _types.SwapCurrencyError)(swapInfo, fromCurrencyCode, toCurrencyCode)
      }

      const {
        addressFrom,
        finishPayment,
        amountResult,
        uid,
        memoFrom
      } = createOrderJson.result

      const toNativeAmount = await toWallet.denominationToNative(
        amountResult.toString(),
        toCurrencyCode
      )

      const spendInfo = {
        currencyCode: fromCurrencyCode,
        spendTargets: [
          {
            nativeAmount: request.nativeAmount,
            publicAddress: addressFrom,
            uniqueIdentifier: memoFrom
          }
        ],
        networkFeeOption: fromCurrencyCode === 'BTC' ? 'high' : 'standard',
        swapData: {
          orderId: uid,
          orderUri: orderUri + uid,
          isEstimate: mode === 'float',
          payoutAddress: toAddress,
          payoutCurrencyCode: toCurrencyCode,
          payoutNativeAmount: toNativeAmount,
          payoutWalletId: request.toWallet.id,
          plugin: { ...swapInfo },
          refundAddress: fromAddress
        }
      }

      return {
        request,
        spendInfo,
        swapInfo,
        fromNativeAmount: request.nativeAmount,
        expirationDate: _swaphelpers.ensureInFuture.call(void 0, finishPayment)
      }
    }

    // Try them all
    try {
      return await getQuote('fix')
    } catch (e) {
      try {
        return await getQuote('float')
      } catch (e2) {
        // Should throw the fixed-rate error
        throw e
      }
    }
  }

  const out = {
    swapInfo,

    async fetchSwapQuote(req) {
      const requestTop = _utils.convertRequest.call(void 0, req)
      const {
        fromCurrencyCode,
        toWallet,
        toCurrencyCode,
        nativeAmount,
        quoteFor
      } = requestTop

      if (quoteFor !== 'to') {
        const newRequest = await _swaphelpers.getMaxSwappable.call(void 0, 
          fetchSwapQuoteInner,
          requestTop
        )
        const swapOrder = await fetchSwapQuoteInner(newRequest)
        return await _swaphelpers.makeSwapPluginQuote.call(void 0, swapOrder)
      } else {
        // Exit early if trade isn't like kind assets
        if (!_swaphelpers.isLikeKind.call(void 0, fromCurrencyCode, toCurrencyCode)) {
          throw new (0, _types.SwapCurrencyError)(
            swapInfo,
            fromCurrencyCode,
            toCurrencyCode
          )
        }
        // Must make a copy of the request because this is a shared object
        // reused between requests to other exchange plugins
        const requestToHack = { ...requestTop }
        const requestToExchangeAmount = await toWallet.nativeToDenomination(
          nativeAmount,
          toCurrencyCode
        )
        let fromQuoteNativeAmount = nativeAmount
        let retries = 5
        while (--retries !== 0) {
          requestToHack.nativeAmount = fromQuoteNativeAmount
          const swapOrder = await fetchSwapQuoteInner(requestToHack)
          if (!('spendInfo' in swapOrder)) break
          if (_optionalChain([swapOrder, 'access', _ => _.spendInfo, 'access', _2 => _2.swapData, 'optionalAccess', _3 => _3.payoutNativeAmount]) == null) break

          const toExchangeAmount = await toWallet.nativeToDenomination(
            _optionalChain([swapOrder, 'access', _4 => _4.spendInfo, 'access', _5 => _5.swapData, 'optionalAccess', _6 => _6.payoutNativeAmount]),
            toCurrencyCode
          )
          if (_biggystring.gte.call(void 0, toExchangeAmount, requestToExchangeAmount)) {
            return await _swaphelpers.makeSwapPluginQuote.call(void 0, swapOrder)
          } else {
            // Get the % difference between the FROM and TO amounts and increase the FROM amount
            // by that %
            const diff = _biggystring.sub.call(void 0, requestToExchangeAmount, toExchangeAmount)
            const percentDiff = _biggystringplus.div18.call(void 0, diff, requestToExchangeAmount)
            const diffMultiplier = _biggystring.add.call(void 0, '1.001', percentDiff)
            fromQuoteNativeAmount = _biggystring.mul.call(void 0, diffMultiplier, fromQuoteNativeAmount)
          }
        }
        throw new (0, _types.SwapCurrencyError)(swapInfo, fromCurrencyCode, toCurrencyCode)
      }
    }
  }
  return out
} exports.makeSwapuzPlugin = makeSwapuzPlugin;






const asApiResponse = (cleaner) =>
  _cleaners.asObject.call(void 0, {
    result: _cleaners.asMaybe.call(void 0, cleaner),
    status: _cleaners.asNumber
  })

const asGetRate = _cleaners.asObject.call(void 0, {
  // result: asNumber,
  // amount: asNumber,
  // rate: asNumber,
  // withdrawFee: asNumber,
  minAmount: _cleaners.asNumber
})

// const asNetwork = asObject({
//   shortName: asString,
//   isDeposit: asBoolean,
//   isWithdraw: asBoolean,
//   isMemo: asBoolean,
//   isActive: asBoolean
// })

const asCreateOrder = _cleaners.asObject.call(void 0, {
  uid: _cleaners.asString,
  // from: asObject({
  //   shortName: asString,
  //   isMemo: asBoolean,
  //   isDeposit: asBoolean,
  //   isWithdraw: asBoolean,
  //   network: asArray(asNetwork)
  // }),
  // to: asObject({
  //   shortName: asString,
  //   isMemo: asBoolean,
  //   isDeposit: asBoolean,
  //   isWithdraw: asBoolean,
  //   network: asArray(asNetwork)
  // }),
  amount: _cleaners.asNumber,
  amountResult: _cleaners.asNumber,
  addressFrom: _cleaners.asString,
  addressTo: _cleaners.asString,
  // addressFromNetwork: asMaybe(asString),
  // addressToNetwork: asString,
  memoFrom: _cleaners.asMaybe.call(void 0, _cleaners.asString),
  // memoTo: asMaybe(asString),
  // createDate: asString,
  finishPayment: _cleaners.asDate
})
